
#include <stdio.h>
#include <string.h>
#ifdef PSP
#include <pspctrl.h>
#endif
#include "script.h"
#include "config.h"
#include "util.h"
#include "def.h"

// configuration variables
int           CPUFreq, RAMFreq, BUSFreq;
RENDER_MODE   RenderMode;
BOOL          VSync;
BOOL          ScreenSmoothing;
int           StateSlot;
BOOL          EnablePSG;
BOOL          EnableYM2413;
int           SoundVolume;
BOOL          RunBios;
char          ThemeFile[MAX_PATH];
char          LangFile[MAX_PATH];
char          ROMFolder[MAX_PATH];
char          SRAMFolder[MAX_PATH];
char          StateFolder[MAX_PATH];
char          CheatFolder[MAX_PATH];
char          ImageFolder[MAX_PATH];
char          ShotFolder[MAX_PATH];
SHOT_FORMAT   ShotFormat;
struct _KEYS  Keys;
BOOL          UseAnalogAsDButton;

// theme
char          BGFile[MAX_PATH];
char          FontsFile[MAX_PATH];
char          FontName[MAX_STRING];
int           FontType;
unsigned long FontColor[2];
unsigned long FontSelectColor[2];
unsigned long FontTitleColor[2];
unsigned long FontInGameColor[2];

// configuration variable names (used in script file)
const char CN_CPUFreq[]            = "CPUFrequency";
const char CN_RAMFreq[]            = "RAMFrequency";
const char CN_BUSFreq[]            = "BUSFrequency";
const char CN_RenderMode[]         = "RenderMode";
const char CN_VSync[]              = "VSync";
const char CN_ScreenSmoothing[]    = "ScreenSmoothing";
const char CN_StateSlot[]          = "StateSlot";
const char CN_EnablePSG[]          = "EnablePSG";
const char CN_EnableYM2413[]       = "EnableYM2413";
const char CN_SoundVolume[]        = "SoundVolume";
const char CN_RunBios[]            = "RunBios";
const char CN_ThemeFile[]          = "ThemeFile";
const char CN_LangFile[]           = "LanguageFile";
const char CN_ROMFolder[]          = "ROMsFolder";
const char CN_SRAMFolder[]         = "SRAMFolder";
const char CN_StateFolder[]        = "StateFolder";
const char CN_CheatFolder[]        = "CheatFolder";
const char CN_ImageFolder[]        = "ImageFolder";
const char CN_ShotFolder[]         = "ShotFolder";
const char CN_ShotFormat[]         = "ShotFormat";
const char CN_KeyMenu[]            = "MenuButton";
const char CN_KeyTurbo[]           = "TurboButton";
const char CN_KeySaveState[]       = "SaveStateButton";
const char CN_KeyLoadState[]       = "LoadStateButton";
const char CN_Key1Up[]             = "Joystick1.Up";
const char CN_Key1Down[]           = "Joystick1.Down";
const char CN_Key1Left[]           = "Joystick1.Left";
const char CN_Key1Right[]          = "Joystick1.Right";
const char CN_Key1One[]            = "Joystick1.One";
const char CN_Key1Two[]            = "Joystick1.Two";
const char CN_Key1OneTwo[]         = "Joystick1.OneTwo";
const char CN_Key1Start[]          = "Joystick1.Start";
const char CN_Key1Reset[]          = "Joystick1.Reset";
const char CN_Key2Up[]             = "Joystick2.Up";
const char CN_Key2Down[]           = "Joystick2.Down";
const char CN_Key2Left[]           = "Joystick2.Left";
const char CN_Key2Right[]          = "Joystick2.Right";
const char CN_Key2One[]            = "Joystick2.One";
const char CN_Key2Two[]            = "Joystick2.Two";
const char CN_Key2OneTwo[]         = "Joystick2.OneTwo";
const char CN_Key2Start[]          = "Joystick2.Start";
const char CN_Key2Reset[]          = "Joystick2.Reset";
const char CN_UseAnalogAsDButton[] = "UseAnalogAsDButton";

// theme
const char CN_BGFile[]           = "BackgroundFile";
const char CN_FontsFile[]        = "FontsFile";
const char CN_FontName[]         = "FontName";
const char CN_FontType[]         = "FontType";
const char CN_FontColor1[]       = "FontColor1";
const char CN_FontColor2[]       = "FontColor2";
const char CN_FontSelectColor1[] = "FontSelectColor1";
const char CN_FontSelectColor2[] = "FontSelectColor2";
const char CN_FontTitleColor1[]  = "FontTitleColor1";
const char CN_FontTitleColor2[]  = "FontTitleColor2";
const char CN_FontInGameColor1[] = "FontInGameColor1";
const char CN_FontInGameColor2[] = "FontInGameColor2";

const char *RenderModesStr[] = { "normal", "fit", "wide", "full" };

// return the key value
static unsigned int get_key(const char *name)
{
  if(!strcmpi(name, "UP"))       return PSP_CTRL_UP;
  if(!strcmpi(name, "DOWN"))     return PSP_CTRL_DOWN;
  if(!strcmpi(name, "LEFT"))     return PSP_CTRL_LEFT;
  if(!strcmpi(name, "RIGHT"))    return PSP_CTRL_RIGHT;

  if(!strcmpi(name, "CROSS")    || !strcmpi(name, "X")) return PSP_CTRL_CROSS;
  if(!strcmpi(name, "CIRCLE")   || !strcmpi(name, "O")) return PSP_CTRL_CIRCLE;
  if(!strcmpi(name, "SQUARE")   || !strcmpi(name, "S")) return PSP_CTRL_SQUARE;
  if(!strcmpi(name, "TRIANGLE") || !strcmpi(name, "T")) return PSP_CTRL_TRIANGLE;
  if(!strcmpi(name, "LTRIGGER") || !strcmpi(name, "L")) return PSP_CTRL_LTRIGGER;
  if(!strcmpi(name, "RTRIGGER") || !strcmpi(name, "R")) return PSP_CTRL_RTRIGGER;

  if(!strcmpi(name, "START"))  return PSP_CTRL_START;
  if(!strcmpi(name, "SELECT")) return PSP_CTRL_SELECT;

  return 0;
}

static const char none_str[] = "NONE";

static void set_key(char *buf, unsigned int key)
{
  switch(key)
  {
    case PSP_CTRL_UP:       strcpy(buf, "UP");       break;
    case PSP_CTRL_DOWN:     strcpy(buf, "DOWN");     break;
    case PSP_CTRL_LEFT:     strcpy(buf, "LEFT");     break;
    case PSP_CTRL_RIGHT:    strcpy(buf, "RIGHT");    break;
    case PSP_CTRL_CROSS:    strcpy(buf, "CROSS");    break;
    case PSP_CTRL_CIRCLE:   strcpy(buf, "CIRCLE");   break;
    case PSP_CTRL_SQUARE:   strcpy(buf, "SQUARE");   break;
    case PSP_CTRL_TRIANGLE: strcpy(buf, "TRIANGLE"); break;
    case PSP_CTRL_LTRIGGER: strcpy(buf, "LTRIGGER"); break;
    case PSP_CTRL_RTRIGGER: strcpy(buf, "RTRIGGER"); break;
    case PSP_CTRL_START:    strcpy(buf, "START");    break;
    case PSP_CTRL_SELECT:   strcpy(buf, "SELECT");   break;
    default:                strcpy(buf, none_str);   break;
  }
}

// load configuration file
void Config_Load(const char *name)
{
  SCRIPT_FILE *f;
  char         buf[MAX_STRING];

  // loading configuration file (if exists)
  f = Script_Load(name);
  // cpu frequency
  CPUFreq = Script_GetInt(f, CN_CPUFreq, 200, (BOOL *)0);
  if     (CPUFreq > 333) CPUFreq = 333;
  else if(CPUFreq < 111) CPUFreq = 111;
  RAMFreq = Script_GetInt(f, CN_RAMFreq, 200, (BOOL *)0);
  if     (RAMFreq > 333) RAMFreq = 333;
  else if(RAMFreq < 111) RAMFreq = 111;
  BUSFreq = Script_GetInt(f, CN_BUSFreq, 111, (BOOL *)0);
  if     (BUSFreq > 166) BUSFreq = 166;
  else if(BUSFreq < 66)  BUSFreq = 66;
  // render mode
  Script_GetString(f, CN_RenderMode, RenderModesStr[1], 0, buf, MAX_STRING, (BOOL *)0);
  if     (!strcmpi(buf, RenderModesStr[3])) RenderMode = RENDER_MODE_FULL;
  else if(!strcmpi(buf, RenderModesStr[2])) RenderMode = RENDER_MODE_WIDE;
  else if(!strcmpi(buf, RenderModesStr[0])) RenderMode = RENDER_MODE_NORMAL;
  else                                      RenderMode = RENDER_MODE_FIT;
  // vertical synchronization
  VSync = Script_GetBool(f, CN_VSync, TRUE, (BOOL *)0);
  // screen smoothing
  ScreenSmoothing = Script_GetBool(f, CN_ScreenSmoothing, TRUE, (BOOL *)0); 
  // state slot
  StateSlot = Script_GetInt(f, CN_StateSlot, 0, (BOOL *)0);
  if     (StateSlot < 0) StateSlot = 0;
  else if(StateSlot > 9) StateSlot = 9;
  // sound emulation
  EnablePSG    = Script_GetBool(f, CN_EnablePSG, TRUE, (BOOL *)0);
  EnableYM2413 = Script_GetBool(f, CN_EnableYM2413, FALSE, (BOOL *)0);
  SoundVolume  = Script_GetInt(f, CN_SoundVolume, 2, (BOOL *)0);
  if(SoundVolume != 1 && SoundVolume != 3 && SoundVolume != 4) SoundVolume = 2;
  // bios
  RunBios      = Script_GetBool(f, CN_RunBios, TRUE, (BOOL *)0);

#ifdef WIN32
  Script_GetString(f, CN_ThemeFile, ".\\theme\\theme.c", 0, ThemeFile, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_LangFile, ".\\language\\english.c", 0, LangFile, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_ROMFolder, ".\\roms\\", 0, ROMFolder, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_SRAMFolder, ".\\sram\\", 0, SRAMFolder, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_StateFolder, ".\\state\\", 0, StateFolder, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_CheatFolder, ".\\cheat\\", 0, CheatFolder, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_ImageFolder, ".\\image\\", 0, ImageFolder, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_ShotFolder, ".\\shot\\", 0, ShotFolder, MAX_PATH, (BOOL *)0);
#elif defined(PSP)
  Script_GetString(f, CN_ThemeFile, "./theme/theme.c", 0, ThemeFile, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_LangFile, "./language/english.c", 0, LangFile, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_ROMFolder, "ms0:/", 0, ROMFolder, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_SRAMFolder, "./sram/", 0, SRAMFolder, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_StateFolder, "./state/", 0, StateFolder, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_CheatFolder, "./cheat/", 0, CheatFolder, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_ImageFolder, "./image/", 0, ImageFolder, MAX_PATH, (BOOL *)0);
  Script_GetString(f, CN_ShotFolder, "ms0:/PSP/PHOTO/", 0, ShotFolder, MAX_PATH, (BOOL *)0);
#endif
  // screen shot format
  Script_GetString(f, CN_ShotFormat, "PNG", 0, buf, MAX_STRING, (BOOL *)0);
  ShotFormat = strcmpi(buf, "BMP") ? SHOT_FORMAT_PNG : SHOT_FORMAT_BMP;
  // keys
  Script_GetString(f, CN_KeyMenu, "TRIANGLE", 0, buf, MAX_STRING, (BOOL *)0);
  Keys.Menu      = get_key(buf);
  // turbo key
  Script_GetString(f, CN_KeyTurbo, none_str, 0, buf, MAX_STRING, (BOOL *)0);
  Keys.Turbo     = get_key(buf);
  // load and save state keys
  Script_GetString(f, CN_KeySaveState, "L", 0, buf, MAX_STRING, (BOOL *)0);
  Keys.SaveState = get_key(buf);
  Script_GetString(f, CN_KeyLoadState, "R", 0, buf, MAX_STRING, (BOOL *)0);
  Keys.LoadState = get_key(buf);
  Script_GetString(f, CN_Key1Up, "UP", 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P1_Up     = get_key(buf);
  Script_GetString(f, CN_Key1Down, "DOWN", 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P1_Down   = get_key(buf);
  Script_GetString(f, CN_Key1Left, "LEFT", 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P1_Left   = get_key(buf);
  Script_GetString(f, CN_Key1Right, "RIGHT", 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P1_Right  = get_key(buf);
  Script_GetString(f, CN_Key1One, "CROSS", 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P1_One    = get_key(buf);
  Script_GetString(f, CN_Key1Two, "CIRCLE", 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P1_Two    = get_key(buf);
  Script_GetString(f, CN_Key1OneTwo, "SQUARE", 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P1_OneTwo = get_key(buf);
  Script_GetString(f, CN_Key1Start, "START", 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P1_Start  = get_key(buf);
  Script_GetString(f, CN_Key1Reset, "SELECT", 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P1_Reset  = get_key(buf);
  Script_GetString(f, CN_Key2Up, none_str, 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P2_Up     = get_key(buf);
  Script_GetString(f, CN_Key2Down, none_str, 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P2_Down   = get_key(buf);
  Script_GetString(f, CN_Key2Left, none_str, 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P2_Left   = get_key(buf);
  Script_GetString(f, CN_Key2Right, none_str, 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P2_Right  = get_key(buf);
  Script_GetString(f, CN_Key2One, none_str, 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P2_One    = get_key(buf);
  Script_GetString(f, CN_Key2Two, none_str, 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P2_Two    = get_key(buf);
  Script_GetString(f, CN_Key2OneTwo, none_str, 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P2_OneTwo = get_key(buf);
  Script_GetString(f, CN_Key2Start, none_str, 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P2_Start  = get_key(buf);
  Script_GetString(f, CN_Key2Reset, none_str, 0, buf, MAX_STRING, (BOOL *)0);
  Keys.P2_Reset  = get_key(buf);
  
  UseAnalogAsDButton = Script_GetBool(f, CN_UseAnalogAsDButton, TRUE, (BOOL *)0);

  Config_LoadTheme(ThemeFile);

  Script_Free(f);
}

BOOL Config_Save(const char *name)
{
  FILE       *f;
  char        buf[MAX_STRING];
  const char *render_mode;
  int         align;

  f = fopen(name, "wt");
  if(!f) return FALSE;

  fprintf(f, "// %s\n\n", "PSPMaster configuration file");

  // cpu clock
  align = sizeof(CN_ScreenSmoothing) - 1;
  Script_WriteIntDirect(f, CN_CPUFreq, CPUFreq, "CPU clock frequency", align);
  Script_WriteIntDirect(f, CN_RAMFreq, RAMFreq, (const char *)0, align);
  Script_WriteIntDirect(f, CN_BUSFreq, BUSFreq, (const char *)0, align);
  // render mode
  switch(RenderMode)
  {
    case RENDER_MODE_FULL:   render_mode = RenderModesStr[3]; break;
    case RENDER_MODE_WIDE:   render_mode = RenderModesStr[2]; break;
    case RENDER_MODE_NORMAL: render_mode = RenderModesStr[0]; break;
    case RENDER_MODE_FIT:
    default:                 render_mode = RenderModesStr[1];  break;
  }
  Script_WriteStringDirect(f, CN_RenderMode, render_mode, 0, (const char *)0, align);
  // vsync
  Script_WriteBoolDirect(f, CN_VSync, VSync, (const char *)0, align);
  // screen smoothing
  Script_WriteBoolDirect(f, CN_ScreenSmoothing, ScreenSmoothing, (const char *)0, align);
  // state slot
  Script_WriteIntDirect(f, CN_StateSlot, StateSlot, (const char *)0, align);

  // sound emulation
  align = sizeof(CN_EnableYM2413) - 1;
  fprintf(f, "\n// %s\n", "sound emulation");
  Script_WriteBoolDirect(f, CN_EnablePSG, EnablePSG, "set to true to enable PSG sound chip emulation", align);
  Script_WriteBoolDirect(f, CN_EnableYM2413, EnableYM2413, "set to true to enable YM2413 sound chip emulation", align);
  Script_WriteIntDirect(f, CN_SoundVolume, SoundVolume, "sound volume: 1x, 2x, 3x or 4x", align);
  // run bios
  Script_WriteBoolDirect(f, CN_RunBios, RunBios, "run bios", align);
  
  // theme file to use
  Script_WriteStringDirect(f, CN_ThemeFile, ThemeFile, 0, "theme", 0);
  // language file to use
  Script_WriteStringDirect(f, CN_LangFile, LangFile, 0, "language file", 0);

  // roms folder
  Script_WriteStringDirect(f, CN_ROMFolder, ROMFolder, 0, "ROMs folder", 0);
  // sram folder
  Script_WriteStringDirect(f, CN_SRAMFolder, SRAMFolder, 0, "folder to save SRAM data", 0);
  // state folder
  Script_WriteStringDirect(f, CN_StateFolder, StateFolder, 0, "folder to save state files", 0);
  // cheat folder
  Script_WriteStringDirect(f, CN_CheatFolder, CheatFolder, 0, "folder to cheats files", 0);
  // images folder
  Script_WriteStringDirect(f, CN_ImageFolder, ImageFolder, 0, "folder for images", 0);
  // screen shot folder
  Script_WriteStringDirect(f, CN_ShotFolder, ShotFolder, 0, "folder to save screen shots", 0);
  // screen shot format
  Script_WriteStringDirect(f, CN_ShotFormat, ShotFormat == SHOT_FORMAT_BMP ? "BMP" : "PNG", 0, "screen shot file format", 0);

  // keys
  align = sizeof(CN_Key1OneTwo) - 1;
  fprintf(f, "\n// %s\n", "Keys");
  set_key(buf, Keys.Menu);
  Script_WriteStringDirect(f, CN_KeyMenu, buf, 0, (const char *)0, align);
  set_key(buf, Keys.Turbo);
  Script_WriteStringDirect(f, CN_KeyTurbo, buf, 0, (const char *)0, align);
  set_key(buf, Keys.SaveState);
  Script_WriteStringDirect(f, CN_KeySaveState, buf, 0, (const char *)0, align);
  set_key(buf, Keys.LoadState);
  Script_WriteStringDirect(f, CN_KeyLoadState, buf, 0, (const char *)0, align);
  // joystick 1
  set_key(buf, Keys.P1_Up);
  Script_WriteStringDirect(f, CN_Key1Up, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P1_Down);
  Script_WriteStringDirect(f, CN_Key1Down, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P1_Left);
  Script_WriteStringDirect(f, CN_Key1Left, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P1_Right);
  Script_WriteStringDirect(f, CN_Key1Right, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P1_One);
  Script_WriteStringDirect(f, CN_Key1One, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P1_Two);
  Script_WriteStringDirect(f, CN_Key1Two, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P1_OneTwo);
  Script_WriteStringDirect(f, CN_Key1OneTwo, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P1_Start);
  Script_WriteStringDirect(f, CN_Key1Start, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P1_Reset);
  Script_WriteStringDirect(f, CN_Key1Reset, buf, 0, (const char *)0, align);
  // joystick 2
  set_key(buf, Keys.P2_Up);
  Script_WriteStringDirect(f, CN_Key2Up, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P2_Down);
  Script_WriteStringDirect(f, CN_Key2Down, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P2_Left);
  Script_WriteStringDirect(f, CN_Key2Left, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P2_Right);
  Script_WriteStringDirect(f, CN_Key2Right, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P2_One);
  Script_WriteStringDirect(f, CN_Key2One, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P2_Two);
  Script_WriteStringDirect(f, CN_Key2Two, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P2_OneTwo);
  Script_WriteStringDirect(f, CN_Key2OneTwo, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P2_Start);
  Script_WriteStringDirect(f, CN_Key2Start, buf, 0, (const char *)0, align);
  set_key(buf, Keys.P2_Reset);
  Script_WriteStringDirect(f, CN_Key2Reset, buf, 0, (const char *)0, align);

  fprintf(f, "\n// %s\n", "use analog button as SMS dbutton");
  Script_WriteBoolDirect(f, CN_UseAnalogAsDButton, UseAnalogAsDButton, (const char *)0, 0);

  fclose(f);

  return TRUE;
}

void Config_LoadTheme(const char *theme)
{
  SCRIPT_FILE *f;
  char         d[MAX_PATH], b[MAX_FILE];

  f = Script_Load(theme);

  if(f)
  {
    char *n;
    strcpy(d, theme);
    n  = GetFileNameFromPath(d);
    *n = 0;
  }
  else *d = 0;

  // background file to use
  Script_GetString(f, CN_BGFile, "bg.png", 0, b, MAX_FILE, (BOOL *)0);
  sprintf(BGFile, "%s%s", d, b);
  // fonts file
  Script_GetString(f, CN_FontsFile, "fonts.bin", 0, b, MAX_FILE, (BOOL *)0);
  sprintf(FontsFile, "%s%s", d, b);
  // font name to use in menu
  Script_GetString(f, CN_FontName, "Out Run", 0, FontName, MAX_STRING, (BOOL *)0);
  // font type and colors to use in menu
  FontType           = Script_GetInt(f, CN_FontType, 1, (BOOL *)0);
  FontColor[0]       = Script_GetInt(f, CN_FontColor1, 0x00FFFFFF, (BOOL *)0);
  FontColor[1]       = Script_GetInt(f, CN_FontColor2, 0x00000000, (BOOL *)0);
  FontSelectColor[0] = Script_GetInt(f, CN_FontSelectColor1, 0x00000000, (BOOL *)0);
  FontSelectColor[1] = Script_GetInt(f, CN_FontSelectColor2, 0x00FFFFFF, (BOOL *)0);
  FontTitleColor[0]  = Script_GetInt(f, CN_FontTitleColor1, 0x00FFFFFF, (BOOL *)0);
  FontTitleColor[1]  = Script_GetInt(f, CN_FontTitleColor2, 0x00000000, (BOOL *)0);
  FontInGameColor[0] = Script_GetInt(f, CN_FontInGameColor1, 0x00FFFFFF, (BOOL *)0);
  FontInGameColor[1] = Script_GetInt(f, CN_FontInGameColor2, 0x00000000, (BOOL *)0);

  if(f) Script_Free(f);
}
