
#ifdef WIN32
#include <windows.h>
#endif
#ifdef SDL
#include "sdl.h"
#endif
#ifdef PSP
#include <pspkernel.h>
#include <pspctrl.h>
#include <psppower.h>
#include <fcntl.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include "zlib.h"
#include "menu.h"
#include "system.h"
#include "video.h"
#include "text.h"
#include "config.h"
#include "shot.h"
#include "language.h"
#include "cheat.h"
#include "util.h"
#include "def.h"

#ifdef PSP
void set_clock_freq(int cpu, int ram, int bus); // main.c
#endif

// menu background image
static VIDEO_BUFFER *bg = (VIDEO_BUFFER *)0;
// buttons image
static VIDEO_BUFFER *btnx = (VIDEO_BUFFER *)0;
static VIDEO_BUFFER *btno = (VIDEO_BUFFER *)0;
static VIDEO_BUFFER *btns = (VIDEO_BUFFER *)0;

// load background and button images
static void load_bg(void)
{
  char path[MAX_PATH], *s;

  bg = Video_LoadImage(BGFile); // try to load image from file
  if(!bg)
  {
    bg = Video_Alloc(480, 272); // if no image, use blank
    if(!bg) exit(0);
  }
  // loading buttons images
  strcpy(path, ImageFolder);
  s = path + strlen(path);
  strcpy(s, "btnx.png");
  btnx = Video_LoadImage(path);
  strcpy(s, "btno.png");
  btno = Video_LoadImage(path);
  strcpy(s, "btns.png");
  btns = Video_LoadImage(path);
}

// release memory used for buttons and background images
static void free_bg(void)
{
  if(bg)   { Video_Free(bg);   bg   = (VIDEO_BUFFER *)0; }
  if(btnx) { Video_Free(btnx); btnx = (VIDEO_BUFFER *)0; }
  if(btno) { Video_Free(btno); btno = (VIDEO_BUFFER *)0; }
  if(btns) { Video_Free(btns); btns = (VIDEO_BUFFER *)0; }
}

typedef struct _MENU_KEYS_STATE {
  BOOL Up, Down, Left, Right;
  BOOL Confirm, Cancel, Exit;
  BOOL L, R;
} MENU_KEYS_STATE;

#ifdef PSP

typedef struct _MENU_KEYS_TIME {
  unsigned long Up, Down, Left, Right;
  unsigned long Confirm, Cancel, Exit;
  unsigned long L, R;
} MENU_KEYS_TIME;

static MENU_KEYS_TIME keys_time[3];
static BOOL           keys_time_init = FALSE;

#endif

static BOOL get_keys_state(MENU_KEYS_STATE *keys)
{
#ifdef SDL

  SDL_Event event;
  BOOL      r = FALSE;

  memset(keys, 0, sizeof(MENU_KEYS_STATE));

  while(SDL_PollEvent(&event))
  {
    switch(event.type)
    {
      case SDL_QUIT:
        exit(0);
        break;

      case SDL_KEYDOWN:
        switch(event.key.keysym.sym)
        {
          case SDLK_UP:     keys->Up      = r = TRUE; break;
          case SDLK_DOWN:   keys->Down    = r = TRUE; break;
          case SDLK_LEFT:   keys->Left    = r = TRUE; break;
          case SDLK_RIGHT:  keys->Right   = r = TRUE; break;
          case SDLK_a:      keys->Confirm = r = TRUE; break;
          case SDLK_s:      keys->Cancel  = r = TRUE; break;
          case SDLK_ESCAPE: keys->Exit    = r = TRUE; break;
          case SDLK_q:      keys->L       = r = TRUE; break;
          case SDLK_w:      keys->R       = r = TRUE; break;
          default: break;
        }
        break;
    }
  }

  return r;

#elif defined(PSP)

  SceCtrlData    psp_pad;
  unsigned long  t1, t2;
  BOOL           r = FALSE;
  int            n;
  const unsigned pb[9] =
  {  PSP_CTRL_UP, PSP_CTRL_DOWN, PSP_CTRL_LEFT, PSP_CTRL_RIGHT, PSP_CTRL_CROSS,
     PSP_CTRL_CIRCLE, PSP_CTRL_SELECT, PSP_CTRL_LTRIGGER, PSP_CTRL_RTRIGGER };
  unsigned long *pt1[9] =
  { &keys_time[0].Up, &keys_time[0].Down, &keys_time[0].Left, &keys_time[0].Right,
    &keys_time[0].Confirm, &keys_time[0].Cancel, &keys_time[0].Exit,
    &keys_time[0].L, &keys_time[0].R };
  unsigned long *pt2[9] =
  { &keys_time[1].Up, &keys_time[1].Down, &keys_time[1].Left, &keys_time[1].Right,
    &keys_time[1].Confirm, &keys_time[1].Cancel, &keys_time[1].Exit,
    &keys_time[1].L, &keys_time[1].R };
  unsigned long *pt3[9] =
  { &keys_time[2].Up, &keys_time[2].Down, &keys_time[2].Left, &keys_time[2].Right,
    &keys_time[2].Confirm, &keys_time[2].Cancel, &keys_time[2].Exit,
    &keys_time[2].L, &keys_time[2].R };
  BOOL          *pk[9] =
  { &keys->Up, &keys->Down, &keys->Left, &keys->Right,
    &keys->Confirm, &keys->Cancel, &keys->Exit,
    &keys->L, &keys->R };

  if(!keys_time_init)
  { memset(keys_time, 0, sizeof(MENU_KEYS_TIME) * 3); keys_time_init = TRUE; }

  sceCtrlPeekBufferPositive(&psp_pad, 1);

  memset(keys, 0, sizeof(MENU_KEYS_STATE));
  for(n = 0; n < 9; n++)
  {
    if(psp_pad.Buttons & pb[n])
    {
      if(*(pt1[n]) == 0)
      {
        *(pt1[n]) = *(pt2[n]) = sceKernelLibcClock();
        *(pt3[n]) = 500 * 1000;
        *(pk[n])  = r = TRUE;
      }
      else
      {
        t1 = sceKernelLibcClock() - *(pt1[n]);
        t2 = sceKernelLibcClock() - *(pt2[n]);
        if(t1 > *(pt3[n]))
        {
          *(pt1[n]) = sceKernelLibcClock();
          *(pk[n])  = r = TRUE;
        }
        if(t2 > 500 * 1000)
        {
          *(pt2[n]) = sceKernelLibcClock();
          *(pt3[n]) >>= 1;
        }
      }
    }
    else *(pt1[n]) = 0;
  }

  return r;
  
#endif
}

static unsigned long line_color[2] = { 0x000000, 0xFFFFFF };

static void set_line_color(unsigned long c1, unsigned long c2)
{
  line_color[0] = c1; line_color[1] = c2;
}

static void draw_line(int x1, int y1, int x2, int y2)
{
  const VIDEO_BUFFER *buf;
  unsigned long      *p;
  int                 dx, dy, x, y, b;

  buf = Video_GetDrawBuffer();

  dx = x2 - x1;
  dy = y2 - y1;
  if(abs(dx) >= abs(dy))
  {
    dy = dy * 512 / dx;
    b  = y1 - (dy * x1 >> 9);
    if(x1 > x2) { x = x1; x1 = x2; x2 = x; }
    for(x = x1; x < x2; x++)
    {
      y = (dy * x >> 9) + b;
      p = (unsigned long *)((unsigned char *)buf->Data + (buf->Pitch * (y - 1))) + x;
      p[-1] = p[0] = p[1] = line_color[1];
      p = (unsigned long *)((unsigned char *)p + buf->Pitch);
      p[-1] = p[0] = p[1] = line_color[1];
      p = (unsigned long *)((unsigned char *)p + buf->Pitch);
      p[-1] = p[0] = p[1] = line_color[1];
    }
    for(x = x1; x < x2; x++)
    {
      y  = (dy * x >> 9) + b;
      p  = (unsigned long *)((unsigned char *)buf->Data + (buf->Pitch * y)) + x;
      *p = line_color[0];
    }
  }
  else
  {
    dx = dx * 512 / dy;
    b  = x1 - (dx * y1 >> 9);
    if(y1 > y2) { y = y1; y1 = y2; y2 = y; }
    for(y = y1; y < y2; y++)
    {
      x = (dx * y >> 9) + b;
      p = (unsigned long *)((unsigned char *)buf->Data + (buf->Pitch * (y - 1))) + x;
      p[-1] = p[0] = p[1] = line_color[1];
      p = (unsigned long *)((unsigned char *)p + buf->Pitch);
      p[-1] = p[0] = p[1] = line_color[1];
      p = (unsigned long *)((unsigned char *)p + buf->Pitch);
      p[-1] = p[0] = p[1] = line_color[1];
    }
    for(y = y1; y < y2; y++)
    {
      x  = (dx * y >> 9) + b;
      p  = (unsigned long *)((unsigned char *)buf->Data + (buf->Pitch * y)) + x;
      *p = line_color[0];
    }
  }
}

// *********************************** MENU OPEN FILE **********************************

// information about an item in the open file menu (can be a file or a folder)
typedef struct _MENU_OPENFILE_ITEM {
  char Name[MAX_FILE]; // item name
  int  Type;           // item type: 0 - file, 1 - folder
} MENU_OPENFILE_ITEM;

// information used by the open file menu
typedef struct _MENU_OPENFILE_DATA {
  char                CurFolder[MAX_PATH]; // current folder path
  MENU_OPENFILE_ITEM *Items;               // items (files and folders)
  int                 NumItems;            // number of items
  int                 NumAlloc;            // number of pointers avaible on "Items"
  int                 Selected;            // current selected file in menu
  int                 FirstItemToDraw;     // first item to draw in the list
} MENU_OPENFILE_DATA;

// language strings for menu open file
typedef struct _MENU_OPENFILE_STRINGS {
  char Select[MAX_STRING];
  char Cancel[MAX_STRING];
  char Exit  [MAX_STRING];
} MENU_OPENFILE_STRINGS;

// information passed to MenuOpenFile_Update function
typedef struct _MENU_OPENFILE_UPDATE {
  MENU_OPENFILE_DATA    *Data;          // data
  MENU_KEYS_STATE       *Keys;          // keys pressed
  MENU_OPENFILE_STRINGS *Strings;       // language strings
  const char           **Filter;        // extension file filter
  int                    FilterItems;   // number of items in the filter
  char                  *FilePath;      // return the file path to open here
  char                  *BaseFolder;
  const char            *Title;         // title
} MENU_OPENFILE_UPDATE;

// init a MENU_OPENFILE_DATA struct
INLINE void MenuOpenFile_InitData(MENU_OPENFILE_DATA *data)
{
  data->Items = (MENU_OPENFILE_ITEM *)malloc(32 * sizeof(MENU_OPENFILE_ITEM));
  if(!data->Items) { exit(0); return; }
  data->NumAlloc = 32;
  data->NumItems = 0;
  data->Selected = data->FirstItemToDraw = 0;
}

// quit memory used by the open file menu data
INLINE void MenuOpenFile_QuitData(MENU_OPENFILE_DATA *data)
{
  free(data->Items);
}

INLINE void MenuOpenFile_ResetData(MENU_OPENFILE_DATA *data)
{
  data->NumItems = data->Selected = 0;
}

INLINE void MenuOpenFile_AddDataItem(MENU_OPENFILE_DATA *data, const char *name, int type)
{
  if(data->NumAlloc <= data->NumItems)
  {
    void *p = realloc(data->Items, (data->NumAlloc + 32) * sizeof(MENU_OPENFILE_ITEM));
    if(!p) { exit(0); return; }
    data->Items     = (MENU_OPENFILE_ITEM *)p;
    data->NumAlloc += 32;
  }
  strncpy(data->Items[data->NumItems].Name, name, MAX_FILE);
  data->Items[data->NumItems++].Type = type;
}

int MenuOpenFile_CompareStrings(const char *str1, const char *str2)
{
  int r;
  while(!(r = *str1 - *str2) && *str1) { str1++; str2++; }
  return r;
}

void MenuOpenFile_FillData(MENU_OPENFILE_DATA *data, const char **filter, int filter_items)
{
  int nf, fok;
  int j;

#ifdef WIN32

  HANDLE          hFind;
  WIN32_FIND_DATA wfd;

  MenuOpenFile_ResetData(data);
  GetCurrentDirectory(MAX_PATH, data->CurFolder);
  MenuOpenFile_AddDataItem(data, "..", 1);

  hFind = FindFirstFile("*.*", &wfd);
  if(hFind != INVALID_HANDLE_VALUE)
  {
    do
    {
      if(wfd.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
      {
        if(strcmpi(wfd.cFileName, ".") && strcmpi(wfd.cFileName, ".."))
          MenuOpenFile_AddDataItem(data, wfd.cFileName, 1);
      }
      else
      {
        char *ext = GetFileExtFromPath(wfd.cFileName);
        if(ext)
        {
          for(nf = fok = 0; nf < filter_items; nf++)
          {
            if(!strcmpi(ext, filter[nf])) { fok = 1; break; }
          }
          if(fok) MenuOpenFile_AddDataItem(data, wfd.cFileName, 0);
        }
      }
    } while(FindNextFile(hFind, &wfd));
    FindClose(hFind);
  }

  j = 2;

#elif defined(PSP)

  SceUID      dir;
  SceIoDirent inf;
  const char  d_root[] = "ms0:/";

  dir = sceIoDopen(data->CurFolder);
  if(dir < 0)
  {
    strcpy(data->CurFolder, d_root);
    dir = sceIoDopen(data->CurFolder);
    if(dir < 0) { exit(0); return; }
  }
  MenuOpenFile_ResetData(data);
  // there is no ".." on the root directory
  if(strcmp(data->CurFolder, d_root)) { MenuOpenFile_AddDataItem(data, "..", 1); j = 2; }
  else j = 1;
  memset(&inf, 0, sizeof(SceIoDirent));
  while(sceIoDread(dir, &inf) > 0)
  {
    if(FIO_S_ISDIR(inf.d_stat.st_mode))
    {
      if(strcmpi(inf.d_name, ".") && strcmpi(inf.d_name, ".."))
      {
        strncat(inf.d_name, "/", 256);
        MenuOpenFile_AddDataItem(data, inf.d_name, 1);
      }
    }
    else
    {
      char *ext = GetFileExtFromPath(inf.d_name);
      if(ext)
      {
        for(nf = fok = 0; nf < filter_items; nf++)
        {
          if(!strcmpi(ext, filter[nf])) { fok = 1; break; }
        }
        if(fok) MenuOpenFile_AddDataItem(data, inf.d_name, 0);
      }
    }
    memset(&inf, 0, sizeof(SceIoDirent));
  }
  sceIoDclose(dir);

#endif

  // put items in order (directories first)
  {
    MENU_OPENFILE_ITEM item;
    BOOL              changed = TRUE;
    int               i;
    while(changed)
    {
      changed = FALSE;
      for(i = j; i < data->NumItems; i++)
      {
        if(data->Items[i - 1].Type == 0 && data->Items[i].Type == 1)
        {
          memcpy(&item, &data->Items[i], sizeof(MENU_OPENFILE_ITEM));
          memcpy(&data->Items[i], &data->Items[i - 1], sizeof(MENU_OPENFILE_ITEM));
          memcpy(&data->Items[i - 1], &item, sizeof(MENU_OPENFILE_ITEM));
          changed = TRUE;
        }
        else if(data->Items[i - 1].Type == data->Items[i].Type)
        {
          if(MenuOpenFile_CompareStrings(data->Items[i - 1].Name, data->Items[i].Name) > 0)
          {
            memcpy(&item, &data->Items[i], sizeof(MENU_OPENFILE_ITEM));
            memcpy(&data->Items[i], &data->Items[i - 1], sizeof(MENU_OPENFILE_ITEM));
            memcpy(&data->Items[i - 1], &item, sizeof(MENU_OPENFILE_ITEM));
            changed = TRUE;
          }
        }
      }
    }
  }
}

/* update open file menu
** return 0 if menu must continue
** return 1 if menu end without opening any file
** return 2 if menu end and must open file */
int MenuOpenFile_Update(MENU_OPENFILE_UPDATE *p)
{
  BOOL update_screen = FALSE;

  if(p->Keys)
  {
    if(p->Keys->Up && p->Data->Selected > 0)
    { p->Data->Selected--; update_screen = TRUE; }
    else if(p->Keys->Down && p->Data->Selected < p->Data->NumItems - 1)
    { p->Data->Selected++; update_screen = TRUE; }
    else if(p->Keys->Confirm && p->Data->Selected < p->Data->NumItems)
    {
      if(p->Data->Items[p->Data->Selected].Type == 1) // user select a folder
      {
        if(p->Data->Selected == 0) // '..'
        {
#ifdef WIN32
          SetCurrentDirectory("..");
#elif defined(PSP)
          if(strcmpi(p->Data->CurFolder, "ms0:/"))
          {
            int l = strlen(p->Data->CurFolder) - 2;
            while(l > 0)
            {
              if(p->Data->CurFolder[l] == '/' || p->Data->CurFolder[l] == '\\')
              { p->Data->CurFolder[l + 1] = 0; break; }
              else l--;
            }
          }
          else strncat(p->Data->CurFolder, p->Data->Items[p->Data->Selected].Name, MAX_PATH);
#endif
          MenuOpenFile_FillData(p->Data, p->Filter, p->FilterItems);
          update_screen = TRUE;
        }
        else
        {
#ifdef WIN32
          SetCurrentDirectory(p->Data->Items[p->Data->Selected].Name);
#elif defined(PSP)
          strncat(p->Data->CurFolder, p->Data->Items[p->Data->Selected].Name, MAX_PATH);
#endif
          MenuOpenFile_FillData(p->Data, p->Filter, p->FilterItems);
          update_screen = TRUE;
        }
      }
      else if(p->Data->Items[p->Data->Selected].Type == 0) // user select a file
      {
#ifdef WIN32
        sprintf(p->FilePath, "%s\\%s", p->Data->CurFolder, p->Data->Items[p->Data->Selected].Name);
        sprintf(p->BaseFolder, "%s\\", p->Data->CurFolder);
#elif defined(PSP)
        sprintf(p->FilePath, "%s%s", p->Data->CurFolder, p->Data->Items[p->Data->Selected].Name);
        strcpy(p->BaseFolder, p->Data->CurFolder);
#endif
        return 2;
      }
    }
    else if(p->Keys->R && p->Data->Selected < p->Data->NumItems - 1)
    {
      p->Data->Selected += 20;
      if(p->Data->Selected >= p->Data->NumItems) p->Data->Selected = p->Data->NumItems - 1;
      update_screen = TRUE;
    }
    else if(p->Keys->L && p->Data->Selected > 0)
    {
      p->Data->Selected -= 20;
      if(p->Data->Selected < 0) p->Data->Selected = 0;
      update_screen = TRUE;
    }
    else if(p->Keys->Cancel) return 1;
    else if(p->Keys->Exit)   exit(0);
  }
  else // first time
  {
#ifdef WIN32
    SetCurrentDirectory(p->BaseFolder);
#elif defined(PSP)
    strcpy(p->Data->CurFolder, p->BaseFolder);
#endif
    MenuOpenFile_FillData(p->Data, p->Filter, p->FilterItems);
    update_screen = TRUE;
  }

  // update screen
  if(update_screen)
  {
    int n, max, y;
    Video_Blit(bg, 0, 0, 480, 272, FALSE);
    Text_SetFontColor(FontType, FontTitleColor[0], FontTitleColor[1]);
    Text_Draw(FontType, p->Title, 0, 4, TDF_CENTER_SCREEN_X);
    while(p->Data->Selected < p->Data->FirstItemToDraw)       p->Data->FirstItemToDraw--;
    while(p->Data->Selected >= p->Data->FirstItemToDraw + 21) p->Data->FirstItemToDraw++;
    max = p->Data->NumItems - p->Data->FirstItemToDraw;
    if(max > 21) max = 21;
    for(n = p->Data->FirstItemToDraw, y = 30; n < p->Data->NumItems; y += 10, n++)
    {
      if(p->Data->Selected == n) Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      else                       Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      if(y < 240) Text_Draw(FontType, p->Data->Items[n].Name, 4, y, 0);
    }

    if(btnx && btno && btns)
    {
      Text_SetFontColor(FontType, FontTitleColor[0], FontTitleColor[1]);
      Video_Blit(btnx, 4, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Select, 24, 256, 0);
      y = 24 + ((2 + strlen(p->Strings->Select)) * 8);
      Video_Blit(btno, y, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Cancel, y + 20, 256, 0);
      y += 20 + ((2 + strlen(p->Strings->Cancel)) * 8);
      Video_Blit(btns, y, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Exit, y + 20, 256, 0);
    }

    Video_Flip(FALSE);
  }

  return 0;
}

BOOL Menu_OpenFile(char *file, char *base_folder, const char *title, const char **filter,
                   int filter_items)
{
  MENU_OPENFILE_UPDATE  p;
  MENU_OPENFILE_DATA    data;
  MENU_KEYS_STATE       keys;
  MENU_OPENFILE_STRINGS str;
  int                   r;
  BOOL                  release_bg = FALSE;
#ifdef WIN32
  char                  save_dir[MAX_PATH];
  GetCurrentDirectory(MAX_PATH, save_dir);
#endif

  Language_GetString("OpenFile_Select", "Select", str.Select, MAX_STRING);
  Language_GetString("OpenFile_Cancel", "Cancel", str.Cancel, MAX_STRING);
  Language_GetString("OpenFile_Exit", "Exit", str.Exit, MAX_STRING);

  // trying to load menu background and buttons images
  if(!bg) { load_bg(); release_bg = TRUE; }

  MenuOpenFile_InitData(&data);
  p.Data        = &data;
  p.Keys        = (MENU_KEYS_STATE *)0;
  p.Strings     = &str;
  p.Filter      = filter;
  p.FilterItems = filter_items;
  p.BaseFolder  = base_folder;
  p.FilePath    = file;
  p.Title       = title;
  MenuOpenFile_Update(&p);

  p.Keys = &keys;
  while(1)
  {
    if(get_keys_state(&keys))
    {
      r = MenuOpenFile_Update(&p);
      if(r) break;
    }

#ifdef SDL
    SDL_Delay(1);
#elif defined(PSP)
    sceKernelDelayThreadCB(1000);
#endif
  }

  MenuOpenFile_QuitData(&data);

#ifdef WIN32
  SetCurrentDirectory(save_dir);
#endif

  if(release_bg) free_bg();

  return (BOOL)(r == 2);
}

// *************************************************************************************



// *********************************** MENU OPEN ROM ***********************************

// display the open ROM menu, acts like the windows open file dialog
BOOL Menu_OpenROM(char *rom_path)
{
  const char *filter[3] = { ".sms", ".gg", ".zip" };
  char        title[MAX_STRING];

  Language_GetString("OpenROM_Title", "Open ROM", title, MAX_STRING);

  return Menu_OpenFile(rom_path, ROMFolder, title, filter, 3);
}

// *************************************************************************************


// *********************************** MENU OPTIONS ************************************

// information used by the options menu
typedef struct _MENU_OPTIONS_DATA {
  int         CPUFreq, RAMFreq, BUSFreq;
  RENDER_MODE Render;
  BOOL        VSync, Smoothing, EnablePSG, EnableYM2413;
  int         SoundVolume;
  SHOT_FORMAT ShotFormat;
  BOOL        RunBios;
  int         CurItem;
} MENU_OPTIONS_DATA;

// strings used in options menu (loaded from the language file)
typedef struct _MENU_OPTIONS_STRINGS {
  int  Align;                    // size of the biggest string
  char Title       [MAX_STRING];
  char CPUFreq     [MAX_STRING];
  char RAMFreq     [MAX_STRING];
  char BUSFreq     [MAX_STRING];
  char Render      [MAX_STRING];
  char VSync       [MAX_STRING];
  char Smoothing   [MAX_STRING];
  char EnablePSG   [MAX_STRING];
  char EnableYM2413[MAX_STRING];
  char SoundVolume [MAX_STRING];
  char ShotFormat  [MAX_STRING];
  char RunBios     [MAX_STRING];
  char RenderNormal[MAX_STRING];
  char RenderFit   [MAX_STRING];
  char RenderWide  [MAX_STRING];
  char RenderFull  [MAX_STRING];
  char Yes         [MAX_STRING];
  char No          [MAX_STRING];
  char Save        [MAX_STRING];
  char Cancel      [MAX_STRING];
  char Exit        [MAX_STRING];
} MENU_OPTIONS_STRINGS;

// information passed to the function MenuOptions_Update
typedef struct _MENU_OPTIONS_UPDATE {
  MENU_OPTIONS_DATA    *Data;
  MENU_KEYS_STATE      *Keys;
  MENU_OPTIONS_STRINGS *Strings;
} MENU_OPTIONS_UPDATE;

int MenuOptions_Update(MENU_OPTIONS_UPDATE *p)
{
  BOOL update_screen = FALSE;

  if(p->Keys)
  {
    if     (p->Keys->Up && p->Data->CurItem > 0)
    { p->Data->CurItem--; update_screen = TRUE; }
    else if(p->Keys->Down && p->Data->CurItem < 10)
    { p->Data->CurItem++; update_screen = TRUE; }
    else if(p->Keys->Left || p->Keys->L)
    {
      switch(p->Data->CurItem)
      {
        case 0:
          update_screen = TRUE;
          if     (p->Data->CPUFreq > 300 && p->Data->CPUFreq <= 333) p->Data->CPUFreq = 300;
          else if(p->Data->CPUFreq > 266 && p->Data->CPUFreq <= 300) p->Data->CPUFreq = 266;
          else if(p->Data->CPUFreq > 222 && p->Data->CPUFreq <= 266) p->Data->CPUFreq = 222;
          else if(p->Data->CPUFreq > 200 && p->Data->CPUFreq <= 222) p->Data->CPUFreq = 200;
          else                                                       p->Data->CPUFreq = 166;
          break;
        case 1:
          update_screen = TRUE;
          if     (p->Data->RAMFreq > 300 && p->Data->RAMFreq <= 333) p->Data->RAMFreq = 300;
          else if(p->Data->RAMFreq > 266 && p->Data->RAMFreq <= 300) p->Data->RAMFreq = 266;
          else if(p->Data->RAMFreq > 222 && p->Data->RAMFreq <= 266) p->Data->RAMFreq = 222;
          else if(p->Data->RAMFreq > 200 && p->Data->RAMFreq <= 222) p->Data->RAMFreq = 200;
          else                                                       p->Data->RAMFreq = 166;
          break;
        case 2:
          update_screen = TRUE;
          if     (p->Data->BUSFreq > 133 && p->Data->BUSFreq <= 166) p->Data->BUSFreq = 133;
          else if(p->Data->BUSFreq > 111 && p->Data->BUSFreq <= 133) p->Data->BUSFreq = 111;
          else if(p->Data->BUSFreq > 99  && p->Data->BUSFreq <= 111) p->Data->BUSFreq = 99;
          else                                                       p->Data->BUSFreq = 66;
          break;
        case 3:
          switch(p->Data->Render)
          { case RENDER_MODE_FULL:   p->Data->Render = RENDER_MODE_WIDE;   update_screen = TRUE; break;
            case RENDER_MODE_WIDE:   p->Data->Render = RENDER_MODE_FIT;    update_screen = TRUE; break;
            case RENDER_MODE_FIT:    p->Data->Render = RENDER_MODE_NORMAL; update_screen = TRUE; break;
            case RENDER_MODE_NORMAL: break; }
          break;
        case 4:
          if(p->Data->VSync) { p->Data->VSync = FALSE; update_screen = TRUE; }
          break;
        case 5:
          if(p->Data->Smoothing) { p->Data->Smoothing = FALSE; update_screen = TRUE; }
          break;
        case 6:
          if(p->Data->EnablePSG) { p->Data->EnablePSG = FALSE; update_screen = TRUE; }
          break;
        case 7:
          if(p->Data->EnableYM2413) { p->Data->EnableYM2413 = FALSE; update_screen = TRUE; }
          break;
        case 8:
          if(p->Data->SoundVolume > 1) { p->Data->SoundVolume--; update_screen = TRUE; }
          break;
        case 9:
          if(p->Data->ShotFormat == SHOT_FORMAT_PNG)
          { p->Data->ShotFormat = SHOT_FORMAT_BMP; update_screen = TRUE; }
          break;
        case 10:
          if(p->Data->RunBios) { p->Data->RunBios = FALSE; update_screen = TRUE; }
          break;
      }
    }
    else if(p->Keys->Right || p->Keys->R)
    {
      switch(p->Data->CurItem)
      {
        case 0:
          update_screen = TRUE;
          if     (p->Data->CPUFreq >= 166 && p->Data->CPUFreq < 200) p->Data->CPUFreq = 200;
          else if(p->Data->CPUFreq >= 200 && p->Data->CPUFreq < 222) p->Data->CPUFreq = 222;
          else if(p->Data->CPUFreq >= 222 && p->Data->CPUFreq < 266) p->Data->CPUFreq = 266;
          else if(p->Data->CPUFreq >= 266 && p->Data->CPUFreq < 300) p->Data->CPUFreq = 300;
          else                                                       p->Data->CPUFreq = 333;
          break;
        case 1:
          update_screen = TRUE;
          if     (p->Data->RAMFreq >= 166 && p->Data->RAMFreq < 200) p->Data->RAMFreq = 200;
          else if(p->Data->RAMFreq >= 200 && p->Data->RAMFreq < 222) p->Data->RAMFreq = 222;
          else if(p->Data->RAMFreq >= 222 && p->Data->RAMFreq < 266) p->Data->RAMFreq = 266;
          else if(p->Data->RAMFreq >= 266 && p->Data->RAMFreq < 300) p->Data->RAMFreq = 300;
          else                                                       p->Data->RAMFreq = 333;
          break;
        case 2:
          update_screen = TRUE;
          if     (p->Data->BUSFreq >= 66  && p->Data->BUSFreq < 99)  p->Data->BUSFreq = 99;
          else if(p->Data->BUSFreq >= 99  && p->Data->BUSFreq < 111) p->Data->BUSFreq = 111;
          else if(p->Data->BUSFreq >= 111 && p->Data->BUSFreq < 133) p->Data->BUSFreq = 133;
          else                                                       p->Data->BUSFreq = 166;
          break;
        case 3:
          switch(p->Data->Render)
          { case RENDER_MODE_NORMAL: p->Data->Render = RENDER_MODE_FIT;    update_screen = TRUE; break;
            case RENDER_MODE_FIT:    p->Data->Render = RENDER_MODE_WIDE;   update_screen = TRUE; break;
            case RENDER_MODE_WIDE:   p->Data->Render = RENDER_MODE_FULL;   update_screen = TRUE; break;
            case RENDER_MODE_FULL:   break; }
          break;
        case 4:
          if(!p->Data->VSync) { p->Data->VSync = TRUE; update_screen = TRUE; }
          break;
        case 5:
          if(!p->Data->Smoothing) { p->Data->Smoothing = TRUE; update_screen = TRUE; }
          break;
        case 6:
          if(!p->Data->EnablePSG) { p->Data->EnablePSG = TRUE; update_screen = TRUE; }
          break;
        case 7:
          if(!p->Data->EnableYM2413) { p->Data->EnableYM2413 = TRUE; update_screen = TRUE; }
          break;
        case 8:
          if(p->Data->SoundVolume < 4) { p->Data->SoundVolume++; update_screen = TRUE; }
          break;
        case 9:
          if(p->Data->ShotFormat == SHOT_FORMAT_BMP)
          { p->Data->ShotFormat = SHOT_FORMAT_PNG; update_screen = TRUE; }
          break;
        case 10:
          if(!p->Data->RunBios) { p->Data->RunBios = TRUE; update_screen = TRUE; }
          break;
      }
    }
    else if(p->Keys->Confirm)
    {
      CPUFreq         = p->Data->CPUFreq;
      RAMFreq         = p->Data->RAMFreq;
      BUSFreq         = p->Data->BUSFreq;
      RenderMode      = p->Data->Render;
      VSync           = p->Data->VSync;
      ScreenSmoothing = p->Data->Smoothing;
      EnablePSG       = p->Data->EnablePSG;
      EnableYM2413    = p->Data->EnableYM2413;
      SoundVolume     = p->Data->SoundVolume;
      ShotFormat      = p->Data->ShotFormat;
      RunBios         = p->Data->RunBios;
      if(EnableYM2413 && !YM2413Init)  { YM2413_Init(MASTER_CLOCK, 44100); YM2413Init = TRUE; }
#ifdef PSP
      set_clock_freq(CPUFreq, RAMFreq, BUSFreq);
#endif
      return 1;
    }
    else if(p->Keys->Cancel) return 1;
    else if(p->Keys->Exit)   exit(0);

  }
  else // menu first time
  {
    p->Data->CPUFreq      = CPUFreq;
    p->Data->RAMFreq      = RAMFreq;
    p->Data->BUSFreq      = BUSFreq;
    p->Data->Render       = RenderMode;
    p->Data->VSync        = VSync;
    p->Data->Smoothing    = ScreenSmoothing;
    p->Data->EnablePSG    = EnablePSG;
    p->Data->EnableYM2413 = EnableYM2413;
    p->Data->SoundVolume  = SoundVolume;
    p->Data->ShotFormat   = ShotFormat;
    p->Data->RunBios      = RunBios;
    p->Data->CurItem      = 0;
    update_screen         = TRUE;
  }

  if(update_screen)
  {
    unsigned long c1, c2;
    int           n, x, y;
    char          buf[16];
    const char   *s;

    Video_Blit(bg, 0, 0, 480, 272, FALSE);

    // title
    Text_SetFontColor(FontType, FontTitleColor[0], FontTitleColor[1]);
    n = (FontType == 1 || FontType == 2) ? 8 : 4;
    Text_Draw(FontType, p->Strings->Title, 0, 4, TDF_CENTER_SCREEN_X);

    x = 4;  // (480 - ((p->Strings->Align + 6) * 8)) >> 1;
    y = 20; // (272 - 70) >> 1;

    // cpu freq
    if(p->Data->CurItem == 0) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->CPUFreq, x, y, 0);
    sprintf(buf, "%i", p->Data->CPUFreq);
    Text_Draw(FontType, buf, (p->Strings->Align * n) + x, y, 0);

    // ram freq
    y += 10;
    if(p->Data->CurItem == 1) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->RAMFreq, x, y, 0);
    sprintf(buf, "%i", p->Data->RAMFreq);
    Text_Draw(FontType, buf, (p->Strings->Align * n) + x, y, 0);

    // bus freq
    y += 10;
    if(p->Data->CurItem == 2) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->BUSFreq, x, y, 0);
    sprintf(buf, "%i", p->Data->BUSFreq);
    Text_Draw(FontType, buf, (p->Strings->Align * n) + x, y, 0);

    // render
    y += 10;
    if(p->Data->CurItem == 3) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->Render, x, y, 0);
    switch(p->Data->Render)
    { case RENDER_MODE_NORMAL: s = p->Strings->RenderNormal; break;
      case RENDER_MODE_WIDE:   s = p->Strings->RenderWide;   break;
      case RENDER_MODE_FULL:   s = p->Strings->RenderFull;   break;
      default:                 s = p->Strings->RenderFit;    break; }
    Text_Draw(FontType, s, (p->Strings->Align * n) + x, y, 0);

    // vsync
    y += 10;
    if(p->Data->CurItem == 4) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->VSync, x, y, 0);
    s = p->Data->VSync ? p->Strings->Yes : p->Strings->No;
    Text_Draw(FontType, s, (p->Strings->Align * n) + x, y, 0);

    // smoothing
    y += 10;
    if(p->Data->CurItem == 5) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->Smoothing, x, y, 0);
    s = p->Data->Smoothing ? p->Strings->Yes : p->Strings->No;
    Text_Draw(FontType, s, (p->Strings->Align * n) + x, y, 0);

    // psg
    y += 10;
    if(p->Data->CurItem == 6) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->EnablePSG, x, y, 0);
    s = p->Data->EnablePSG ? p->Strings->Yes : p->Strings->No;
    Text_Draw(FontType, s, (p->Strings->Align * n) + x, y, 0);

    // ym2413
    y += 10;
    if(p->Data->CurItem == 7) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->EnableYM2413, x, y, 0);
    s = p->Data->EnableYM2413 ? p->Strings->Yes : p->Strings->No;
    Text_Draw(FontType, s, (p->Strings->Align * n) + x, y, 0);

    // sound volume
    y += 10;
    if(p->Data->CurItem == 8) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->SoundVolume, x, y, 0);
    sprintf(buf, "%ix", p->Data->SoundVolume);
    Text_Draw(FontType, buf, (p->Strings->Align * n) + x, y, 0);

    // shot format
    y += 10;
    if(p->Data->CurItem == 9) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->ShotFormat, x, y, 0);
    s = p->Data->ShotFormat == SHOT_FORMAT_BMP ? "BMP" : "PNG";
    Text_Draw(FontType, s, (p->Strings->Align * n) + x, y, 0);

    // run bios
    y += 10;
    if(p->Data->CurItem == 10) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                       { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->RunBios, x, y, 0);
    s = p->Data->RunBios ? p->Strings->Yes : p->Strings->No;
    Text_Draw(FontType, s, (p->Strings->Align * n) + x, y, 0);

    if(btnx && btno && btns)
    {
      Text_SetFontColor(FontType, FontTitleColor[0], FontTitleColor[1]);
      Video_Blit(btnx, 4, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Save, 24, 256, 0);
      x = 24 + ((2 + strlen(p->Strings->Save)) * 8);
      Video_Blit(btno, x, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Cancel, x + 20, 256, 0);
      x += 20 + ((2 + strlen(p->Strings->Cancel)) * 8);
      Video_Blit(btns, x, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Exit, x + 20, 256, 0);
    }

    Video_Flip(FALSE);
  }

  return 0;
}

void Menu_Options(void)
{
  MENU_OPTIONS_UPDATE  p;
  MENU_OPTIONS_DATA    data;
  MENU_KEYS_STATE      keys;
  MENU_OPTIONS_STRINGS str;
  int                  i, j;
  BOOL                 release_bg   = FALSE;
  const char          *str_name[21] = {
    "Options_Title", "Options_CPUFreq", "Options_RAMFreq", "Options_BUSFreq", "Options_Render",
    "Options_VSync", "Options_Smoothing", "Options_EnablePSG", "Options_EnableYM2413",
    "Options_SoundVolume", "Options_ShotFormat", "Options_RunBios", "Options_NormalRender",
    "Options_FitRender", "Options_WideRender", "Options_FullRender", "Options_Yes", "Options_No",
    "Options_Save", "Options_Cancel", "Options_Exit" };
  const char          *str_def[21]  = {
    "Options", "CPU Frequency:", "RAM Frequency:", "BUS Frequency:", "Render:",
    "VSync:", "Screen Smoothing:", "Enable PSG Sound:", "Enable YM2413 Sound:", "Sound Volume:",
    "Screen Shot File Format:", "Run BIOS:", "Normal", "Fit", "Wide", "Full", "Yes", "No",
    "Save", "Cancel", "Exit" };
  char                *str_buf[21]  = {
    str.Title, str.CPUFreq, str.RAMFreq, str.BUSFreq, str.Render, str.VSync, str.Smoothing,
    str.EnablePSG, str.EnableYM2413, str.SoundVolume, str.ShotFormat, str.RunBios, str.RenderNormal,
    str.RenderFit, str.RenderWide, str.RenderFull, str.Yes, str.No, str.Save, str.Cancel, str.Exit };

  // trying to load menu background
  if(!bg) { load_bg(); release_bg = TRUE; }

  // loading strings from the language file
  for(i = str.Align = 0; i < 21; i++)
  {
    j = Language_GetString(str_name[i], str_def[i], str_buf[i], MAX_STRING);
    if(i > 0 && i < 12 && j > str.Align) str.Align = j;
  }

  p.Data    = &data;
  p.Keys    = (MENU_KEYS_STATE *)0;
  p.Strings = &str;
  MenuOptions_Update(&p);

  p.Keys = &keys;
  while(1)
  {
    if(get_keys_state(&keys))
    {
      if(MenuOptions_Update(&p)) break;
    }

#ifdef SDL
    SDL_Delay(1);
#elif defined(PSP)
    sceKernelDelayThreadCB(1000);
#endif
  }

  if(release_bg) free_bg();
}

// *************************************************************************************


// ********************************* SELECT LANG MENU **********************************

BOOL Menu_GetLangFile(char *lang_path)
{
  const char *filter[1] = { ".c" };
#ifdef WIN32
  char        base_folder[MAX_PATH] = ".\\language";
#elif defined(PSP)
  char        base_folder[MAX_PATH] = "./language/";
#endif
  char        title[MAX_STRING];

  Language_GetString("OpenLanguage_Title", "Open Language File", title, MAX_STRING);

  return Menu_OpenFile(lang_path, base_folder, title, filter, 1);
}

// *************************************************************************************


// ************************************** MENU CHEATS **********************************

typedef struct _MENU_CHEATS_DATA {
  int Align;           // size of the biggest cheat name
  int Selected;        // current selected file in menu
  int FirstItemToDraw; // first item to draw in the list
} MENU_CHEATS_DATA;

typedef struct _MENU_CHEATS_STRINGS {
  char Title         [MAX_STRING];
  char Enabled       [MAX_STRING];
  char Disabled      [MAX_STRING];
  char EnableDisable [MAX_STRING];
  char Return        [MAX_STRING];
  char Exit          [MAX_STRING];
  char NoCheatAvaible[MAX_STRING];
} MENU_CHEATS_STRINGS;

typedef struct _MENU_CHEATS_UPDATE {
  MENU_CHEATS_DATA    *Data;
  MENU_KEYS_STATE     *Keys;
  MENU_CHEATS_STRINGS *Strings;
} MENU_CHEATS_UPDATE;
 
int MenuCheats_Update(MENU_CHEATS_UPDATE *p)
{
  BOOL update_screen = FALSE;

  if(p->Keys)
  {
    if(p->Keys->Up && p->Data->Selected > 0)
    { p->Data->Selected--; update_screen = TRUE; }
    else if(p->Keys->Down && p->Data->Selected < NumCheats - 1)
    { p->Data->Selected++; update_screen = TRUE; }
    else if(p->Keys->Left && p->Data->Selected < NumCheats)
    {
      if(!Cheats[p->Data->Selected].Enabled)
      { Cheats[p->Data->Selected].Enabled = TRUE; update_screen = TRUE; }
    }
    else if(p->Keys->Right && p->Data->Selected < NumCheats)
    {
      if(Cheats[p->Data->Selected].Enabled)
      { Cheats[p->Data->Selected].Enabled = FALSE; update_screen = TRUE; }
    }
    else if(p->Keys->Confirm && p->Data->Selected < NumCheats)
    {
      Cheats[p->Data->Selected].Enabled = !Cheats[p->Data->Selected].Enabled;
      update_screen = TRUE;
    }
    else if(p->Keys->R && p->Data->Selected < NumCheats - 1)
    {
      p->Data->Selected += 20;
      if(p->Data->Selected >= NumCheats) p->Data->Selected = NumCheats - 1;
      update_screen = TRUE;
    }
    else if(p->Keys->L && p->Data->Selected > 0)
    {
      p->Data->Selected -= 20;
      if(p->Data->Selected < 0) p->Data->Selected = 0;
      update_screen = TRUE;
    }
    else if(p->Keys->Cancel) return 1;
    else if(p->Keys->Exit)   exit(0);
  }
  else update_screen = TRUE; // menu first time

  if(update_screen)
  {
    int  n, max, y, a, l;
    char buf[MAX_STRING + 4], *s;

    Video_Blit(bg, 0, 0, 480, 272, FALSE); // redraw background image

    // menu title
    Text_SetFontColor(FontType, FontTitleColor[0], FontTitleColor[1]);
    Text_Draw(FontType, p->Strings->Title, 0, 4, TDF_CENTER_SCREEN_X);

    if(!Cheats || !NumCheats)
    {
      Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      Text_Draw(FontType, p->Strings->NoCheatAvaible, 0, 0, TDF_CENTER_SCREEN_X | TDF_CENTER_SCREEN_Y);
    }
    else
    {
      while(p->Data->Selected < p->Data->FirstItemToDraw)       p->Data->FirstItemToDraw--;
      while(p->Data->Selected >= p->Data->FirstItemToDraw + 21) p->Data->FirstItemToDraw++;
      max = NumCheats - p->Data->FirstItemToDraw;
      if(max > 21) max = 21;
      for(n = p->Data->FirstItemToDraw, y = 30; n < NumCheats; y += 10, n++)
      {
        if(p->Data->Selected == n) Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
        else                       Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
        if(y < 240)
        {
          Text_Draw(FontType, Cheats[n].Name, 4, y, 0);
          l = Cheats[n].NameLen;
          a = p->Data->Align - l;
          memset(buf, '.', a);
          buf[a] = 0;
          Text_Draw(FontType, buf, 4 + (l * 8), y, 0);
          if(Cheats[n].Enabled) s = p->Strings->Enabled;
          else                  s = p->Strings->Disabled;
          Text_Draw(FontType, s, 4 + ((l + a) * 8), y, 0);
        }
      }
    }

    if(btnx && btno && btns)
    {
      Text_SetFontColor(FontType, FontTitleColor[0], FontTitleColor[1]);
      Video_Blit(btnx, 4, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->EnableDisable, 24, 256, 0);
      y = 24 + ((2 + strlen(p->Strings->EnableDisable)) * 8);
      Video_Blit(btno, y, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Return, y + 20, 256, 0);
      y += 20 + ((2 + strlen(p->Strings->Return)) * 8);
      Video_Blit(btns, y, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Exit, y + 20, 256, 0);
    }

    Video_Flip(FALSE);
  }

  return 0;
}

void Menu_Cheats(void)
{
  MENU_CHEATS_UPDATE  p;
  MENU_CHEATS_DATA    data;
  MENU_KEYS_STATE     keys;
  MENU_CHEATS_STRINGS str;
  int                 i, l;
  BOOL                release_bg = FALSE;
  const char         *str_name[7] =
  { "Cheats_Title", "Cheats_Enabled", "Cheats_Disabled", "Cheats_EnableDisable",
    "Cheats_Return", "Cheats_Exit", "Cheats_NoCheatAvaible" };
  const char         *str_def[7] =
  { "Cheats", "Enabled", "Disabled", "Enable/Disable Cheat", "Return", "Exit",
    "There is no cheats avaible for this game" };
  char               *str_buf[7] =
  { str.Title, str.Enabled, str.Disabled, str.EnableDisable, str.Return, str.Exit, str.NoCheatAvaible };

  // trying to load menu background
  if(!bg) { load_bg(); release_bg = TRUE; }
  
  // loading language strings
  for(i = 0; i < 7; i++) Language_GetString(str_name[i], str_def[i], str_buf[i], MAX_PATH);

  data.Align = data.FirstItemToDraw = data.Selected = 0;
  for(i = 0; i < NumCheats; i++)
  {
    l = strlen(Cheats[i].Name);
    if(l > data.Align) data.Align = l;
  }
  data.Align += 2;

  p.Data    = &data;
  p.Keys    = (MENU_KEYS_STATE *)0;
  p.Strings = &str;
  MenuCheats_Update(&p);

  p.Keys = &keys;
  while(1)
  {
    if(get_keys_state(&keys))
    {
      if(MenuCheats_Update(&p)) break;
    }
  }

  // free memory used to store background
  if(release_bg) free_bg();
}

// *************************************************************************************


// **************************** MENU CONFIGURE KEYS ************************************

typedef struct _MENU_KEYS_CONFIG_DATA {
  unsigned L, Up, Left, Right, Down;
  unsigned R, Triangle, Circle, Square, Cross;
  unsigned Select, Start;
  int      Swap, Change;
  unsigned SwapKey, OrigKey;
  int      CurItem;
} MENU_KEYS_CONFIG_DATA;

typedef struct _MENU_KEYS_CONFIG_STRINGS {
  char Menu       [MAX_STRING];
  char Turbo      [MAX_STRING];
  char SaveState  [MAX_STRING];
  char LoadState  [MAX_STRING];
  char P1_Up      [MAX_STRING];
  char P1_Down    [MAX_STRING];
  char P1_Left    [MAX_STRING];
  char P1_Right   [MAX_STRING];
  char P1_Button1 [MAX_STRING];
  char P1_Button2 [MAX_STRING];
  char P1_Button12[MAX_STRING];
  char P1_Start   [MAX_STRING];
  char P1_Reset   [MAX_STRING];
  char P2_Up      [MAX_STRING];
  char P2_Down    [MAX_STRING];
  char P2_Left    [MAX_STRING];
  char P2_Right   [MAX_STRING];
  char P2_Button1 [MAX_STRING];
  char P2_Button2 [MAX_STRING];
  char P2_Button12[MAX_STRING];
  char P2_Start   [MAX_STRING];
  char P2_Reset   [MAX_STRING];
  char None       [MAX_STRING];
  char Title      [MAX_STRING];
  char Save       [MAX_STRING];
  char Cancel     [MAX_STRING];
  char Exit       [MAX_STRING];
} MENU_KEYS_CONFIG_STRINGS;

typedef struct _MENU_KEYS_CONFIG_UPDATE {
  MENU_KEYS_CONFIG_DATA    *Data;
  MENU_KEYS_STATE          *Keys;
  MENU_KEYS_CONFIG_STRINGS *Strings;
  VIDEO_BUFFER             *PSPImage;
} MENU_KEYS_CONFIG_UPDATE;

static unsigned __no_key;

static unsigned *ck1[23] = {
  &Keys.Menu, &Keys.Turbo, &Keys.SaveState, &Keys.LoadState,
  &Keys.P1_Up, &Keys.P1_Down, &Keys.P1_Left, &Keys.P1_Right,
  &Keys.P1_One, &Keys.P1_Two, &Keys.P1_OneTwo, &Keys.P1_Start, &Keys.P1_Reset,
  &Keys.P2_Up, &Keys.P2_Down, &Keys.P2_Left, &Keys.P2_Right,
  &Keys.P2_One, &Keys.P2_Two, &Keys.P2_OneTwo, &Keys.P2_Start, &Keys.P2_Reset, &__no_key };
static const unsigned ck2[13] = {
  PSP_CTRL_LTRIGGER, PSP_CTRL_UP, PSP_CTRL_LEFT, PSP_CTRL_RIGHT, PSP_CTRL_DOWN,
  PSP_CTRL_RTRIGGER, PSP_CTRL_TRIANGLE, PSP_CTRL_CIRCLE, PSP_CTRL_SQUARE, PSP_CTRL_CROSS,
  PSP_CTRL_SELECT, PSP_CTRL_START };

int MenuKeysConfig_Update(MENU_KEYS_CONFIG_UPDATE *p)
{
  BOOL update_screen = FALSE;

  if(p->Keys)
  {
    unsigned *cur_item = ((unsigned *)p->Data + p->Data->CurItem);

    if(p->Keys->Up && p->Data->CurItem > 0)
    {
      p->Data->CurItem--;
      p->Data->Swap   = -1;
      p->Data->Change = 0;
      update_screen   = TRUE;
    }
    else if(p->Keys->Down && p->Data->CurItem < 11)
    {
      p->Data->CurItem++;
      p->Data->Swap   = -1;
      p->Data->Change = 0;
      update_screen   = TRUE;
    }
    else if(p->Keys->Left && *cur_item > 0)
    {
      unsigned *ck3 = (unsigned *)p->Data;
      int       i;

      if(!p->Data->Change) { p->Data->OrigKey = *cur_item; p->Data->Change = 1; }
      if(p->Data->Swap > -1)
      {
        ck3[p->Data->Swap] = p->Data->SwapKey;
        p->Data->Swap      = -1;
      }
      (*cur_item)--;
      for(i = 0; i < 12; i++)
      {
        if(i != p->Data->CurItem && ck3[i] == *cur_item)
        {
          p->Data->Swap    = i;
          p->Data->SwapKey = ck3[i];
          ck3[i]           = p->Data->OrigKey;
          break;
        }
      }
      update_screen = TRUE;
    }
    else if(p->Keys->Right && *cur_item < 22)
    {
      unsigned *ck3 = (unsigned *)p->Data;
      int       i;

      if(!p->Data->Change) { p->Data->OrigKey = *cur_item; p->Data->Change = 1; }
      if(p->Data->Swap > -1)
      {
        ck3[p->Data->Swap] = p->Data->SwapKey;
        p->Data->Swap      = -1;
      }
      (*cur_item)++;
      for(i = 0; i < 12; i++)
      {
        if(i != p->Data->CurItem && ck3[i] == *cur_item)
        {
          p->Data->Swap    = i;
          p->Data->SwapKey = ck3[i];
          ck3[i]           = p->Data->OrigKey;
          break;
        }
      }
      update_screen = TRUE;
    }
    else if(p->Keys->Confirm)
    {
      unsigned *ck3 = (unsigned *)p->Data;
      int       i;

      memset(&Keys, 0, sizeof(struct _KEYS));
      for(i = 0; i < 12; i++) *ck1[ck3[i]] = ck2[i];

      return 1;
    }
    else if(p->Keys->Cancel) return 1;
    else if(p->Keys->Exit)   exit(0);
  }
  else // menu first time
  {
    unsigned *ck3 = (unsigned *)p->Data;
    unsigned  i, j;

    p->Data->L = p->Data->Up = p->Data->Left = p->Data->Right = p->Data->Down = 22;
    p->Data->R = p->Data->Triangle = p->Data->Circle = p->Data->Square = p->Data->Cross = 22;
    p->Data->Start = p->Data->Select = 22;
    p->Data->Swap = -1;
    p->Data->Change = p->Data->CurItem = 0;
    for(i = 0; i < 22; i++)
    {
      for(j = 0; j < 12; j++) if(*ck1[i] == ck2[j]) ck3[j] = i;
    }

    update_screen = TRUE;
  }

  if(update_screen)
  {
    char *s;

    // redraw background image
    Video_Blit(bg, 0, 0, 480, 272, FALSE);
    // draw PSP image
    Video_Blit(p->PSPImage, (480 - 300) / 2, (272 - 130) / 2, 0, 0, TRUE);

    Text_SetFontColor(FontType, FontTitleColor[0], FontTitleColor[1]);
    Text_Draw(FontType, p->Strings->Title, 0, 4, TDF_CENTER_SCREEN_X);

    if(p->Data->CurItem == 0)
    { Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      set_line_color(FontSelectColor[0], FontSelectColor[1]); }
    else
    { Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      set_line_color(FontColor[0], FontColor[1]); }
    draw_line(115, 60, 135, 80);
    s = p->Strings->Menu + (p->Data->L * MAX_STRING);
    Text_Draw(FontType, s, 107, 58, TDF_RIGHT);

    if(p->Data->CurItem == 1)
    { Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      set_line_color(FontSelectColor[0], FontSelectColor[1]); }
    else
    { Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      set_line_color(FontColor[0], FontColor[1]); }
    s = p->Strings->Menu + (p->Data->Up * MAX_STRING);
    Text_Draw(FontType, s, 128, 104, TDF_CENTER);

    if(p->Data->CurItem == 2)
    { Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      set_line_color(FontSelectColor[0], FontSelectColor[1]); }
    else
    { Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      set_line_color(FontColor[0], FontColor[1]); }
    s = p->Strings->Menu + (p->Data->Left * MAX_STRING);
    Text_Draw(FontType, s, 107, 129, TDF_RIGHT);

    if(p->Data->CurItem == 3)
    { Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      set_line_color(FontSelectColor[0], FontSelectColor[1]); }
    else
    { Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      set_line_color(FontColor[0], FontColor[1]); }
    s = p->Strings->Menu + (p->Data->Right * MAX_STRING);
    Text_Draw(FontType, s, 150, 124, 0);

    if(p->Data->CurItem == 4)
    { Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      set_line_color(FontSelectColor[0], FontSelectColor[1]); }
    else
    { Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      set_line_color(FontColor[0], FontColor[1]); }
    s = p->Strings->Menu + (p->Data->Down * MAX_STRING);
    Text_Draw(FontType, s, 128, 156, TDF_CENTER);

    if(p->Data->CurItem == 5)
    { Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      set_line_color(FontSelectColor[0], FontSelectColor[1]); }
    else
    { Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      set_line_color(FontColor[0], FontColor[1]); }
    draw_line(370, 60, 350, 80);
    s = p->Strings->Menu + (p->Data->R * MAX_STRING);
    Text_Draw(FontType, s, 378, 58, 0);

    if(p->Data->CurItem == 6)
    { Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      set_line_color(FontSelectColor[0], FontSelectColor[1]); }
    else
    { Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      set_line_color(FontColor[0], FontColor[1]); }
    s = p->Strings->Menu + (p->Data->Triangle * MAX_STRING);
    Text_Draw(FontType, s, 358, 100, TDF_CENTER);

    if(p->Data->CurItem == 7)
    { Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      set_line_color(FontSelectColor[0], FontSelectColor[1]); }
    else
    { Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      set_line_color(FontColor[0], FontColor[1]); }
    s = p->Strings->Menu + (p->Data->Circle * MAX_STRING);
    Text_Draw(FontType, s, 384, 129, 0);

    if(p->Data->CurItem == 8)
    { Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      set_line_color(FontSelectColor[0], FontSelectColor[1]); }
    else
    { Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      set_line_color(FontColor[0], FontColor[1]); }
    s = p->Strings->Menu + (p->Data->Square * MAX_STRING);
    Text_Draw(FontType, s, 336, 134, TDF_RIGHT);

    if(p->Data->CurItem == 9)
    { Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      set_line_color(FontSelectColor[0], FontSelectColor[1]); }
    else
    { Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      set_line_color(FontColor[0], FontColor[1]); }
    s = p->Strings->Menu + (p->Data->Cross * MAX_STRING);
    Text_Draw(FontType, s, 358, 158, TDF_CENTER);

    if(p->Data->CurItem == 10)
    { Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      set_line_color(FontSelectColor[0], FontSelectColor[1]); }
    else
    { Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      set_line_color(FontColor[0], FontColor[1]); }
    draw_line(308, 198, 288, 218);
    s = p->Strings->Menu + (p->Data->Select * MAX_STRING);
    Text_Draw(FontType, s, 280, 216, TDF_RIGHT);

    if(p->Data->CurItem == 11)
    { Text_SetFontColor(FontType, FontSelectColor[0], FontSelectColor[1]);
      set_line_color(FontSelectColor[0], FontSelectColor[1]); }
    else
    { Text_SetFontColor(FontType, FontColor[0], FontColor[1]);
      set_line_color(FontColor[0], FontColor[1]); }
    draw_line(328, 198, 348, 218);
    s = p->Strings->Menu + (p->Data->Start * MAX_STRING);
    Text_Draw(FontType, s, 356, 216, 0);

    if(btnx && btno && btns)
    {
      int x;
      Text_SetFontColor(FontType, FontTitleColor[0], FontTitleColor[1]);
      Video_Blit(btnx, 4, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Save, 24, 256, 0);
      x = 24 + ((2 + strlen(p->Strings->Save)) * 8);
      Video_Blit(btno, x, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Cancel, x + 20, 256, 0);
      x += 20 + ((2 + strlen(p->Strings->Cancel)) * 8);
      Video_Blit(btns, x, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Exit, x + 20, 256, 0);
    }

    Video_Flip(FALSE);
  }

  return 0;
}

void Menu_KeysConfig(void)
{
  MENU_KEYS_CONFIG_UPDATE  p;
  MENU_KEYS_CONFIG_DATA    data;
  MENU_KEYS_STATE          keys;
  MENU_KEYS_CONFIG_STRINGS str;
  int                      i;
  BOOL                     release_bg = FALSE;
  const char              *str_name[27] =
  { "KeysConfig_Title", "KeysConfig_None", "KeysConfig_Menu", "KeysConfig_Turbo",
    "KeysConfig_SaveState", "KeysConfig_LoadState", "KeysConfig_P1Up", "KeysConfig_P1Down",
    "KeysConfig_P1Left", "KeysConfig_P1Right", "KeysConfig_P1Button1", "KeysConfig_P1Button2",
    "KeysConfig_P1Button12", "KeysConfig_P1Start", "KeysConfig_P1Reset", "KeysConfig_P2Up",
    "KeysConfig_P2Down", "KeysConfig_P2Left", "KeysConfig_P2Right", "KeysConfig_P2Button1",
    "KeysConfig_P2Button2", "KeysConfig_P2Button12", "KeysConfig_P2Start", "KeysConfig_P2Reset",
    "KeysConfig_Save", "KeysConfig_Cancel", "KeysConfig_Exit" };
  const char              *str_def[27] =
  { "Configure Keys", "None", "Menu", "Turbo", "Save State", "Load State", "P1 Up", "P1 Down",
    "P1 Left", "P1 Right", "P1 Button 1", "P1 Button 2", "P1 Button 1+2", "P1 Start/Pause",
    "P1 Reset", "P2 Up", "P2 Down", "P2 Left", "P2 Right", "P2 Button 1", "P2 Button 2",
    "P2 Button 1+2", "P2 Start/Pause", "P2 Reset", "Save", "Cancel", "Exit" };
  char                    *str_buf[27] =
  { str.Title, str.None, str.Menu, str.Turbo, str.SaveState, str.LoadState, str.P1_Up,
    str.P1_Down, str.P1_Left, str.P1_Right, str.P1_Button1, str.P1_Button2, str.P1_Button12,
    str.P1_Start, str.P1_Reset, str.P2_Up, str.P2_Down, str.P2_Left, str.P2_Right,
    str.P2_Button1, str.P2_Button2, str.P2_Button12, str.P2_Start, str.P2_Reset,
    str.Save, str.Cancel, str.Exit };

  // trying to load the PSP image
  {
    char path[MAX_PATH + 16];
    sprintf(path, "%s%s", ImageFolder, "psp.png");
    p.PSPImage = Video_LoadImage(path);
    if(!p.PSPImage) return;
  }

  // trying to load menu background
  if(!bg) { load_bg(); release_bg = TRUE; }

  // loading language strings
  for(i = 0; i < 27; i++) Language_GetString(str_name[i], str_def[i], str_buf[i], MAX_STRING);

  p.Data    = &data;
  p.Keys    = (MENU_KEYS_STATE *)0;
  p.Strings = &str;
  MenuKeysConfig_Update(&p);

  p.Keys = &keys;
  while(1)
  {
    if(get_keys_state(&keys))
    {
      if(MenuKeysConfig_Update(&p)) break;
    }
  }

  // free memory used to store background
  if(release_bg) free_bg();

  // free psp image
  Video_Free(p.PSPImage);
}


// *************************************************************************************

// ************************************** MAIN MENU ************************************

// main menu data
typedef struct _MENU_MAIN_DATA {
  int CurItem;
} MENU_MAIN_DATA;

typedef struct _MENU_MAIN_STRINGS {
  char OpenROM       [MAX_STRING];
  char Options       [MAX_STRING];
  char ConfigureKeys [MAX_STRING];
  char SaveState     [MAX_STRING];
  char LoadState     [MAX_STRING];
  char StateSlot     [MAX_STRING];
  char ScreenShot    [MAX_STRING];
  char SelectLangFile[MAX_STRING];
  char Cheats        [MAX_STRING];
  char ReturnToGame  [MAX_STRING];
  char Exit          [MAX_STRING];
  char Enter         [MAX_STRING];
  char Cancel        [MAX_STRING];
} MENU_MAIN_STRINGS;

typedef struct _MENU_MAIN_UPDATE {
  MENU_MAIN_DATA    *Data;
  MENU_KEYS_STATE   *Keys;
  MENU_MAIN_STRINGS *Strings;
} MENU_MAIN_UPDATE;

void MenuMain_UpdateLang(MENU_MAIN_STRINGS *str)
{
  int i;

  const char  *str_name[13] =
  { "Menu_OpenROM", "Menu_Options", "Menu_ConfigureKeys", "Menu_SaveState", "Menu_LoadState",
    "Menu_StateSlot", "Menu_ScreenShot", "Menu_SelectLangFile", "Menu_Cheats", "Menu_ReturnToGame",
    "Menu_Exit", "Menu_Enter", "Menu_Cancel" };
  const char  *str_def[13] =
  { "Open ROM", "Options", "Configure Keys", "Save State", "Load State", "State Slot:",
    "Screen Shot",  "Select Language File", "Cheats", "Return to Game", "Exit", "Enter", "Cancel" };
  char        *str_buf[13] =
  { str->OpenROM, str->Options, str->ConfigureKeys, str->SaveState, str->LoadState, str->StateSlot,
    str->ScreenShot, str->SelectLangFile, str->Cheats, str->ReturnToGame, str->Exit, str->Enter,
    str->Cancel };

  // loading language strings
  for(i = 0; i < 13; i++) Language_GetString(str_name[i], str_def[i], str_buf[i], MAX_PATH);
}

int MenuMain_Update(MENU_MAIN_UPDATE *p)
{
  BOOL update_screen = FALSE;

  if(p->Keys)
  {
    if     (p->Keys->Up && p->Data->CurItem > 0)
    { p->Data->CurItem--; update_screen = TRUE; }
    else if(p->Keys->Down && p->Data->CurItem < 10)
    { p->Data->CurItem++; update_screen = TRUE; }
    else if(p->Keys->Left && p->Data->CurItem == 5 && StateSlot > 0)
    { StateSlot--; update_screen = TRUE; }
    else if(p->Keys->Right && p->Data->CurItem == 5 && StateSlot < 9)
    { StateSlot++; update_screen = TRUE; }
    else if(p->Keys->Confirm)
    {
      if(p->Data->CurItem == 0) // open ROM
      {
        char path[MAX_PATH];
        if(Menu_OpenROM(path))
        {
          System_Quit();
          if(System_Init(path))
          {
            char str[MAX_STRING], msg[MAX_STRING + MAX_FILE];
            Language_GetString("Message_ROMLoadOK", "ROM \"%s\" loaded", str, MAX_STRING);
            sprintf(msg, str, Cart.Name);
            Video_ShowMessage(msg, DEFAULT_MESSAGE_TIME);
            return 1;
          }
        }
        update_screen = TRUE;
      }
      if(p->Data->CurItem == 1) // options
      {
        Menu_Options();
        update_screen = TRUE;
      }
      else if(p->Data->CurItem == 2) // configure keys
      {
        Menu_KeysConfig();
        update_screen = TRUE;
      }
      else if(p->Data->CurItem == 3) // save state
      {
#if MAX_PATH > MAX_STRING
        char fn [MAX_PATH];
#else
        char fn [MAX_STRING];
#endif
        char msg[MAX_STRING];
        if(System.Flags & SF_BIOS)
        {
          Language_GetString("Message_CantSaveStateInBios", "Can't save state while in BIOS", msg, MAX_STRING);
          Video_ShowMessage(msg, DEFAULT_MESSAGE_TIME);
        }
        else
        {
          sprintf(fn, "%s%.8lX.st%i", StateFolder, Cart.CRC, StateSlot);
          if(System_SaveState(fn))
            Language_GetString("Message_SaveState", "State %i saved", msg, MAX_STRING);
          else
            Language_GetString("Message_UnableToSaveState", "Unable to save state %i", msg, MAX_STRING);
          sprintf(fn, msg, StateSlot);
          Video_ShowMessage(fn, DEFAULT_MESSAGE_TIME);
        }
        return 1;
      }
      else if(p->Data->CurItem == 4) // load state
      {
#if MAX_PATH > MAX_STRING
        char fn [MAX_PATH];
#else
        char fn [MAX_STRING];
#endif
        char msg[MAX_STRING];
        sprintf(fn, "%s%.8lX.st%i", StateFolder, Cart.CRC, StateSlot);
        if(System_LoadState(fn))
          Language_GetString("Message_LoadState", "State %i loaded", msg, MAX_STRING);
        else
          Language_GetString("Message_UnableToLoadState", "Unable to load state %i", msg, MAX_STRING);
        sprintf(fn, msg, StateSlot);
        Video_ShowMessage(fn, DEFAULT_MESSAGE_TIME);
        return 1;
      }
      else if(p->Data->CurItem == 6 && Cart_Loaded) // take screen shot
      {
        const char *e;
        int         n;

#ifdef WIN32

        char sdir[MAX_PATH];
        char path[MAX_FILE];

        GetCurrentDirectory(MAX_PATH, sdir);
        CreateDirectory(ShotFolder, NULL);
        SetCurrentDirectory(ShotFolder);
        CreateDirectory(Cart.Name, NULL);
        SetCurrentDirectory(Cart.Name);
        n = 0;
        e = ShotFormat == SHOT_FORMAT_BMP ? "bmp" : "png";
        do {
          if(++n > 999) return 0;
          sprintf(path, "shot%.3i.%s", n, e);
        } while(GetFileAttributes(path) != 0xFFFFFFFF);
        if(ShotFormat == SHOT_FORMAT_BMP) Shot_SaveBMP(path);
        else                              Shot_SavePNG(path);
        SetCurrentDirectory(sdir);

#elif defined(PSP)

        SceIoStat stat;
        char      path[MAX_PATH + 32], *d;

        strcpy(path, ShotFolder);
        n = strlen(path) - 1;
        if(path[n] == '/' || path[n] == '\\') path[n--] = 0;
        sceIoMkdir(path, 0777);
        path[++n] = '/';
        strcpy(path + n + 1, Cart.Name);
        sceIoMkdir(path, 0777);
        d = path + n + strlen(Cart.Name) + 1;
        n = 0;
        e = ShotFormat == SHOT_FORMAT_PNG ? "png" : "bmp";
        do
        {
          sprintf(d, "/shot%.3i.%s", n, e);
          if(n > 999) return 0;
          n++;
        } while(sceIoGetstat(path, &stat) >= 0);
        if(ShotFormat == SHOT_FORMAT_PNG) Shot_SavePNG(path);
        else                              Shot_SaveBMP(path);

#endif
      }
      else if(p->Data->CurItem == 7) // select language file
      {
        char path[MAX_PATH];
        if(Menu_GetLangFile(path))
        {
          if(Language_Load(path))
          {
            strcpy(LangFile, path);
            MenuMain_UpdateLang(p->Strings);
          }
        }
        update_screen = TRUE;
      }
      else if(p->Data->CurItem == 8) // cheats menu
      {
        Menu_Cheats();
        update_screen = TRUE;
      }
      else if(p->Data->CurItem == 9) return 1;
      else if(p->Data->CurItem == 10) exit(0);
    }
    else if(p->Keys->Cancel) return 1;
    else if(p->Keys->Exit) exit(0);
  }
  else
  {
    p->Data->CurItem = 0;
    update_screen    = TRUE;
  }

  if(update_screen)
  {
    unsigned long c1, c2;
    int           y;
    char          buf[16];

    Video_Blit(bg, 0, 0, 480, 272, FALSE);

    Text_SetFontColor(FontType, FontTitleColor[0], FontTitleColor[1]);
    Text_Draw(FontType, "PSPMaster v0.08b", 0, 4, TDF_CENTER_SCREEN_X);

    y = 30;
    if(p->Data->CurItem == 0) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->OpenROM, 4, y, 0);

    y += 10;
    if(p->Data->CurItem == 1) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->Options, 4, y, 0);

    y += 10;
    if(p->Data->CurItem == 2) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->ConfigureKeys, 4, y, 0);

    y += 10;
    if(p->Data->CurItem == 3) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->SaveState, 4, y, 0);

    y += 10;
    if(p->Data->CurItem == 4) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->LoadState, 4, y, 0);

    y += 10;
    if(p->Data->CurItem == 5) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->StateSlot, 4, y, 0);
    sprintf(buf, "%i", StateSlot);
    Text_Draw(FontType, buf, (strlen(p->Strings->StateSlot) + 1) * 8, y, 0);

    y += 10;
    if(p->Data->CurItem == 6) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->ScreenShot, 4, y, 0);

    y += 10;
    if(p->Data->CurItem == 7) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->SelectLangFile, 4, y, 0);

    y += 10;
    if(p->Data->CurItem == 8) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->Cheats, 4, y, 0);

    y += 10;
    if(p->Data->CurItem == 9) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                      { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->ReturnToGame, 4, y, 0);

    y += 10;
    if(p->Data->CurItem == 10) { c1 = FontSelectColor[0]; c2 = FontSelectColor[1]; }
    else                       { c1 = FontColor[0];       c2 = FontColor[1];       }
    Text_SetFontColor(FontType, c1, c2);
    Text_Draw(FontType, p->Strings->Exit, 4, y, 0);

    if(btnx && btno && btns)
    {
      Text_SetFontColor(FontType, FontTitleColor[0], FontTitleColor[1]);
      Video_Blit(btnx, 4, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Enter, 24, 256, 0);
      y = 24 + ((2 + strlen(p->Strings->Enter)) * 8);
      Video_Blit(btno, y, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Cancel, y + 20, 256, 0);
      y += 20 + ((2 + strlen(p->Strings->Cancel)) * 8);
      Video_Blit(btns, y, 252, 16, 16, TRUE);
      Text_Draw(FontType, p->Strings->Exit, y + 20, 256, 0);
    }

    Video_Flip(FALSE);
  }

  return 0;
}

// start the emulator main menu
void Menu_Main(void)
{
  MENU_MAIN_UPDATE  p;
  MENU_MAIN_DATA    data;
  MENU_KEYS_STATE   keys;
  MENU_MAIN_STRINGS str;
  BOOL              release_bg = FALSE;

  // trying to load menu background
  if(!bg) { load_bg(); release_bg = TRUE; }
  
  // loading language strings
  MenuMain_UpdateLang(&str);

  p.Data    = &data;
  p.Keys    = (MENU_KEYS_STATE *)0;
  p.Strings = &str;
  MenuMain_Update(&p);

  p.Keys = &keys;
  while(1)
  {
    if(get_keys_state(&keys))
    {
      if(MenuMain_Update(&p)) break;
    }
  }

  // free memory used to store background
  if(release_bg) free_bg();
}

// *************************************************************************************


