
#include <string.h>
#include "video.h"
#include "system.h"
#include "def.h"

#define BACKDROP_COLOR (0x10 | (VDP.Reg[7] & 0x0F))

#ifdef PSP

extern unsigned char  BGNameDirty[0x200];
extern unsigned short BGNameList[0x200];
extern unsigned short BGListIndex;
extern unsigned char  BGPatternCache[0x20000];

extern unsigned char  LUT[0x10000];
extern unsigned long  BPLUT[0x10000];

extern const unsigned long AtexMask[4];

#else

unsigned short Render_Palette[0x100];

unsigned char LUT[0x10000];
unsigned long BPLUT[0x10000];

unsigned short SMSColorTable[0x100];
struct _GG_COLOR GGColorTable[0x100];

unsigned char  BGNameDirty[0x200];
unsigned short BGNameList[0x200];
unsigned short BGListIndex;
unsigned char  BGPatternCache[0x20000];

const unsigned long AtexMask[4] = { 0x00000000, 0x10101010, 0x20202020, 0x30303030, };

#endif

BOOL TablesOK = FALSE;

void Render_InitLUT(void)
{
  int i, j;
  int bx, sx, b, s, bp, bf, sf, c;

  for(bx = 0; bx < 0x100; bx++)
  {
    for(sx = 0; sx < 0x100; sx++)
    {
      b  = (bx & 0x0F);
      bp = (bx & 0x20) ? 1 : 0;
      bf = (bx & 0x7F);
      s  = (sx & 0x0F);
      sf = (sx & 0x0F) | 0x10 | 0x40;
      if(bx & 0x40) c = bf;
      else
      {
        if(bp)
        {
          if(b) c = bf | 0x40;
          else
          {
            if(s) c = sf;
            else  c = bf;
          }
        }
        else
        {
          if(s) c = sf;
          else  c = bf;
        }
      }
      LUT[(bx << 8) | (sx)] = c;
    }
  }

  for(i = 0; i < 0x100; i++)
  {
    for(j = 0; j < 0x100; j++)
    {
      int           x;
      unsigned long out = 0;
      for(x = 0; x < 8; x++)
      {
        out |= (j & (0x80 >> x)) ? (unsigned long)(8 << (x << 2)) : 0;
        out |= (i & (0x80 >> x)) ? (unsigned long)(4 << (x << 2)) : 0;
      }
      BPLUT[(j << 8) | (i)] = out;
    }
  }
}

void Render_InitSMSColorTable(void)
{
  unsigned long c, r, g, b;

  for(c = 0; c < 256; c++)
  {
    r = ((((c >> 0) & 3) << 6) * 5) >> 5;
    g = ((((c >> 2) & 3) << 6) * 5) >> 4;
    b = ((((c >> 4) & 3) << 6) * 5) >> 5;
    if(r > 0x1F) r = 0x1F;
    if(g > 0x3F) g = 0x3F;
    if(b > 0x1F) b = 0x1F;
    SMSColorTable[c] = (unsigned short)((r) | (g << 5) | (b << 11));
  }
}

void Render_InitGGColorTable(void)
{
  unsigned short c, r, g, b;

  for(c = 0; c < 256; c++)
  {
    r = ((((c >> 1) & 7) << 5) * 5) >> 5;
    g = ((((c >> 5) & 7) << 5) * 5) >> 4;
    b = ((((c >> 1) & 7) << 5) * 5) >> 5;
    if(r > 0x1F) r = 0x1F;
    if(g > 0x3F) g = 0x3F;
    if(b > 0x1F) b = 0x1F;
    GGColorTable[c].rg = (unsigned short)(r | (g << 5));
    GGColorTable[c].b  = (unsigned short)(b << 11);
  }
}

void Render_Init(void)
{
  if(!TablesOK)
  {
    Render_InitLUT();
    Render_InitSMSColorTable();
    Render_InitGGColorTable();
    TablesOK = TRUE;
  }

  memset(Render_Palette, 0, 0x100 * 2);
  memset(BGNameDirty, 0, 0x200);
  memset(BGNameList, 0, 0x200 * 2);
  memset(BGPatternCache, 0, 0x20000);
  BGListIndex = 0;
}

#ifndef PSP

void Render_UpdateBgCache(void)
{
  unsigned long i;
  unsigned long x, y;
  unsigned long name;

  if(!BGListIndex) return;

  for(i = 0; i < BGListIndex; i++)
  {
    name = BGNameList[i];
    BGNameList[i] = 0;

    for(y = 0; y < 8; y++)
    {
      if(BGNameDirty[name] & (1 << y))
      {
        unsigned char *dst  = &BGPatternCache[name << 6];
        unsigned short bp01 = *(unsigned short *)&VDP.VRAM[(name << 5) | (y << 2) | (0)];
        unsigned short bp23 = *(unsigned short *)&VDP.VRAM[(name << 5) | (y << 2) | (2)];
        unsigned long  temp = (BPLUT[bp01] >> 2) | (BPLUT[bp23]);

        for(x = 0; x < 8; x++)
        {
          unsigned char c = (unsigned char)((temp >> (x << 2)) & 0x0F);
          dst[0x00000 | (y << 3) | (x)]           = (c);
          dst[0x08000 | (y << 3) | (x ^ 7)]       = (c);
          dst[0x10000 | ((y ^ 7) << 3) | (x)]     = (c);
          dst[0x18000 | ((y ^ 7) << 3) | (x ^ 7)] = (c);
        }
      }
    }
    BGNameDirty[name] = 0;
  }
  BGListIndex = 0;
}

void Render_SMSBgLine(unsigned char *linebuf)
{
  int             v_line      = (VDP.Line + VDP.Reg[9]) % 224;
  int             v_row       = (v_line & 7) << 3;
  int             hscroll     = ((VDP.Reg[0] & 0x40) && (VDP.Line < 0x10)) ? 0 : (0x100 - VDP.Reg[8]);
  int             column      = 0;
  int             nt_scroll   = (hscroll >> 3);
  int             shift       = (hscroll & 7);
  unsigned short *nt          = (unsigned short *)&VDP.VRAM[VDP.ntab + ((v_line >> 3) << 6)];
  unsigned long  *linebuf_ptr = (unsigned long *)(linebuf - shift);
  unsigned short  attr;

  if(shift)
  {
    int x;
    for(x = shift; x < 8; x++) linebuf[x - shift] = 0;
    column++;
  }

  if((((unsigned)linebuf_ptr) & 0x03) == 0)
  {
    unsigned long *dst, *cache_ptr, atex_mask;

    for( ; column < 24; column++)
    {
      attr      = nt[(column + nt_scroll) & 0x1F];
      atex_mask = AtexMask[(attr >> 11) & 3];
      cache_ptr = (unsigned long *)(BGPatternCache + (((attr & 0x7FF) << 6) | (v_row)));
      dst       = linebuf_ptr + (column << 1);
      *dst++    = *cache_ptr++ | atex_mask;
      *dst      = *cache_ptr   | atex_mask;
    }
    if(VDP.Reg[0] & 0x80)
    {
      v_row = (VDP.Line & 7) << 3;
      nt    = (unsigned short *)&VDP.VRAM[((VDP.Reg[2] << 10) & 0x3800) + ((VDP.Line >> 3) << 6)];
    }
    for( ; column < 32; column++)
    {
      attr      = nt[(column + nt_scroll) & 0x1F];
      atex_mask = AtexMask[(attr >> 11) & 3];
      cache_ptr = (unsigned long *)(BGPatternCache + (((attr & 0x7FF) << 6) | (v_row)));
      dst       = linebuf_ptr + (column << 1);
      *dst++    = *cache_ptr++ | atex_mask;
      *dst      = *cache_ptr   | atex_mask;
    }
  }
  else if((((unsigned)linebuf_ptr) & 0x01) == 0)
  {
    unsigned short *dst, *cache_ptr, atex_mask;

    for( ; column < 24; column++)
    {
      attr      = nt[(column + nt_scroll) & 0x1F];
      atex_mask = (unsigned short)AtexMask[(attr >> 11) & 3];
      cache_ptr = (unsigned short *)(BGPatternCache + (((attr & 0x7FF) << 6) | (v_row)));
      dst       = (unsigned short *)(linebuf_ptr + (column << 1));
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst      = *cache_ptr   | atex_mask;
    }
    if(VDP.Reg[0] & 0x80)
    {
      v_row = (VDP.Line & 7) << 3;
      nt    = (unsigned short *)&VDP.VRAM[((VDP.Reg[2] << 10) & 0x3800) + ((VDP.Line >> 3) << 6)];
    }
    for( ; column < 32; column++)
    {
      attr      = nt[(column + nt_scroll) & 0x1F];
      atex_mask = (unsigned short)AtexMask[(attr >> 11) & 3];
      cache_ptr = (unsigned short *)(BGPatternCache + (((attr & 0x7FF) << 6) | (v_row)));
      dst       = (unsigned short *)(linebuf_ptr + (column << 1));
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst      = *cache_ptr   | atex_mask;
    }
  }
  else
  {
    unsigned char *dst, *cache_ptr, atex_mask;

    for( ; column < 24; column++)
    {
      attr      = nt[(column + nt_scroll) & 0x1F];
      atex_mask = (unsigned char)AtexMask[(attr >> 11) & 3];
      cache_ptr = (unsigned char *)(BGPatternCache + (((attr & 0x7FF) << 6) | (v_row)));
      dst       = (unsigned char *)(linebuf_ptr + (column << 1));
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst      = *cache_ptr   | atex_mask;
    }
    if(VDP.Reg[0] & 0x80)
    {
      v_row = (VDP.Line & 7) << 3;
      nt    = (unsigned short *)&VDP.VRAM[((VDP.Reg[2] << 10) & 0x3800) + ((VDP.Line >> 3) << 6)];
    }
    for( ; column < 32; column++)
    {
      attr      = nt[(column + nt_scroll) & 0x1F];
      atex_mask = (unsigned char)AtexMask[(attr >> 11) & 3];
      cache_ptr = (unsigned char *)(BGPatternCache + (((attr & 0x7FF) << 6) | (v_row)));
      dst       = (unsigned char *)(linebuf_ptr + (column << 1));
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst      = *cache_ptr   | atex_mask;
    }
  }

  if(shift)
  {
    int   x, c, a;
    char *p = (char *)&linebuf[(column << 3) - shift];
    attr    = nt[(column + nt_scroll) & 0x1F];
    a       = (attr >> 7) & 0x30;
    for(x = 0; x < shift; x++)
    {
      c = BGPatternCache[((attr & 0x7FF) << 6) | (v_row) | (x)];
      p[x] = ((c) | (a));
    }
  }
}

#endif

void Render_GGBgLine(unsigned char *linebuf)
{
  int             v_line      = (VDP.Line + VDP.Reg[9]) % 224;
  int             v_row       = (v_line & 7) << 3;
  int             hscroll     = 0x100 - VDP.Reg[8];
  unsigned long  *linebuf_ptr = (unsigned long *)(linebuf - (hscroll & 7));
  unsigned short *nt          = (unsigned short *)&VDP.VRAM[VDP.ntab + ((v_line >> 3) << 6)];
  int             column;
  unsigned short  attr;
  int             nt_scroll = (hscroll >> 3);

  if((((unsigned)linebuf_ptr) & 0x03) == 0)
  {
    unsigned long *dst, *cache_ptr, atex_mask;

    for(column = 6; column <= 26; column++)
    {
      attr      = nt[(column + nt_scroll) & 0x1F];
      atex_mask = AtexMask[(attr >> 11) & 3];
      cache_ptr = (unsigned long *)(BGPatternCache + (((attr & 0x7FF) << 6) | (v_row)));
      dst       = linebuf_ptr + (column << 1);
      *dst++    = *cache_ptr++ | atex_mask;
      *dst      = *cache_ptr   | atex_mask;
    }
  }
  else if((((unsigned)linebuf_ptr) & 0x01) == 0)
  {
    unsigned short *dst, *cache_ptr, atex_mask;

    for(column = 6; column <= 26; column++)
    {
      attr      = nt[(column + nt_scroll) & 0x1F];
      atex_mask = (unsigned short)AtexMask[(attr >> 11) & 3];
      cache_ptr = (unsigned short *)(BGPatternCache + (((attr & 0x7FF) << 6) | (v_row)));
      dst       = (unsigned short *)(linebuf_ptr + (column << 1));
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst      = *cache_ptr   | atex_mask;
    }
  }
  else
  {
    unsigned char *dst, *cache_ptr, atex_mask;

    for(column = 6; column <= 26; column++)
    {
      attr      = nt[(column + nt_scroll) & 0x1F];
      atex_mask = (unsigned char)AtexMask[(attr >> 11) & 3];
      cache_ptr = (unsigned char *)(BGPatternCache + (((attr & 0x7FF) << 6) | (v_row)));
      dst       = (unsigned char *)(linebuf_ptr + (column << 1));
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst++    = *cache_ptr++ | atex_mask;
      *dst      = *cache_ptr   | atex_mask;
    }
  }
}

void Render_ObjLine(unsigned char *linebuf)
{
  int            line    = VDP.Line;
  int            count   = 0;
  int            width   = 8;
  int            height  = (VDP.Reg[1] & 0x02) ? 16 : 8;
  unsigned char *st      = (unsigned char *)&VDP.VRAM[VDP.satb];
  int            i;

  if(VDP.Reg[1] & 0x01) { width <<= 1; height <<= 1; }

  for(i = 0; i < 64; i++)
  {
    int yp = st[i];
    if(yp == 208) return;
    yp++;
    if(yp > 240) yp -= 256;
    if((line >= yp) && (line < (yp + height)))
    {
      unsigned char *linebuf_ptr;
      int            start = 0;
      int            end   = width;
      int            xp    = st[0x80 + (i << 1)];
      int            n     = st[0x81 + (i << 1)];

      count++;
      if(VDP.Limit && count == 9) return;
      if(VDP.Reg[0] & 0x08) xp -= 8;
      if(VDP.Reg[6] & 0x04) n |= 0x0100;
      if(VDP.Reg[1] & 0x02) n &= 0x01FE;
      linebuf_ptr = linebuf + xp;
      if(xp < 0) start = (0 - xp);
      if((xp + width) > 256) end = (256 - xp);

      if(VDP.Reg[1] & 0x01)
      {
        unsigned char *cache_ptr = (unsigned char *)&BGPatternCache[(n << 6) | (((line - yp) >> 1) << 3)];
        int            x;

        for(x = start; x < end; x++)
        {
          unsigned char sp = cache_ptr[(x >> 1)];
          if(sp)
          {
            unsigned char bg = linebuf_ptr[x];
            linebuf_ptr[x] = LUT[(bg << 8) | (sp)];
            if(bg & 0x40) VDP.Status |= 0x20;
          }
        }
      }
      else
      {
        unsigned char *cache_ptr = (unsigned char *)&BGPatternCache[(n << 6) | ((line - yp) << 3)];
        int            x;

        for(x = start; x < end; x++)
        {
          unsigned char sp = cache_ptr[x];
          if(sp)
          {
            unsigned char bg = linebuf_ptr[x];
            linebuf_ptr[x] = LUT[(bg << 8) | (sp)];
            if(bg & 0x40) VDP.Status |= 0x20;
          }
        }
      }
    }
  }
}

void Render_GGLine(void)
{
  unsigned long      *d;
  unsigned char      *l;
  int                 i;
  const VIDEO_BUFFER *screen;

  if(VDP.Line < 24 || VDP.Line >= 168) return;
  Render_UpdateBgCache();
  screen = Video_GetScreenBuffer();
  l = (unsigned char *)screen->Data + (screen->Pitch * VDP.Line);
  Render_GGBgLine(l);
  Render_ObjLine(l);
  if(VDP.Reg[0] & 0x20) memset(l, BACKDROP_COLOR, 8);
  for(i = 12, d = (unsigned long *)(l + 48); i < (208 / 4); i++) *d++ &= 0x1F1F1F1F;
}

void Render_SMSLine(void)
{
  unsigned long      *d;
  unsigned char      *l;
  int                 i;
  const VIDEO_BUFFER *screen;

  if(VDP.Line >= 192) return;
  Render_UpdateBgCache();
  screen = Video_GetScreenBuffer();
  l = (unsigned char *)screen->Data + (screen->Pitch * VDP.Line);
  if(!(VDP.Reg[1] & 0x40) || !(VDP.Reg[2] & 0x01))
    memset(l, BACKDROP_COLOR, 256);
  else
  {
    Render_SMSBgLine(l);
    Render_ObjLine(l);
    if(VDP.Reg[0] & 0x20) memset(l, BACKDROP_COLOR, 8);
    for(i = 0, d = (unsigned long *)l; i < (256 / 4); i++) *d++ &= 0x1F1F1F1F;
  }
}
