
#ifndef _SYSTEM_H
#define _SYSTEM_H

#include "z80.h"
#include "vdp.h"
#include "render.h"
#include "psg.h"
#include "ym2413.h"
#include "cart.h"
#include "def.h"

#define MASTER_CLOCK        3579545
#define LINES_PER_FRAME     262
#define FRAMES_PER_SECOND   60
#define CYCLES_PER_LINE     ((MASTER_CLOCK / FRAMES_PER_SECOND) / LINES_PER_FRAME)

#ifdef __cplusplus
extern "C" {
#endif

#define SF_PAUSED     0x01
#define SF_GAME_GEAR  0x02
#define SF_BIOS       0x04

struct _SYSTEM {
  unsigned char RAM[0x2000];
  unsigned char FCR[4];
  unsigned char Port3F;
  unsigned char PortF2;
  unsigned char YM2413_Latch;
  unsigned char Flags;
} extern System;

extern BOOL System_On;
extern BOOL YM2413Init;

struct _JOYPAD
{
  BOOL Up, Down, Left, Right;
  BOOL One, Two, Start, Reset;
} extern Joypad[2];

BOOL System_Init(const char *);
void System_Quit(void);
void System_Reset(BOOL);
BOOL System_Frame(BOOL);
BOOL System_SaveState(const char *);
BOOL System_LoadState(const char *);

void System_MapZ80Mem(word Addr, byte Value);

#ifdef __cplusplus
}
#endif

#endif // _SYSTEM_H
