
#ifdef WIN32
#include <windows.h>
#endif
#include <stdio.h>
#include "util.h"

#if !defined(WIN32) && defined(PSP)
unsigned long __temp_num = 0;
#endif

char *GetTempFilePath(char *buf)
{
#ifdef WIN32
  char d[MAX_PATH - 12];
  if(GetTempPath(sizeof(d), d) > sizeof(d)) { d[0] = '.'; d[1] = 0; }
  GetTempFileName(d, "tmp", 0, buf);
#elif defined(PSP)
  sprintf(buf, "./%s/%.8lX.tmp", "tmp", __temp_num++);
#else
  tmpnam(buf);
#endif
  return buf;
}

// get extension from a file path
char *GetFileExtFromPath(const char *path)
{
  char *s;
  int   c, f;

  if(!path) return 0; // invalid parameter

  s = (char *)path;
  while(*s) s++;

  for(f = 0; s != path; s--)
  {
    c = *s;
    if     (c == '.')              { f = 1; break; }
    else if(c == '\\' || c == '/') { break; }
  }

  if(!f) s = (char *)0;

  return s;
}

// get name from a file path
char *GetFileNameFromPath(const char *path)
{
  char *s;
  int   c;

  if(!path) return 0; // invalid parameter

  s = (char *)path;
  while(*s) s++;

  for( ; s != path - 1; s--)
  {
    c = *s;
    if(c == '\\' || c == '/') break;
  }

  return s + 1;
}

// remove spaces from the begining and the end of the string
int RemoveExtremeSpaces(char *str)
{
  char *s;
  int   c, i, l;

  if(!str) return 0;

  s = str - 1;
  while((c = *(++s))) { if(!ISSPACE(c)) break; }

  l = i = 0;
  if(s != str)
  {
    while((c = *(str++) = *(s++)))
    {
      l++;
      if(ISSPACE(c)) i++;
      else           i = 0;
    }
  }
  else
  {
    while((c = *(str++)))
    {
      l++;
      if(ISSPACE(c)) i++;
      else           i = 0;
    }
  }
  *(str - i - 1) = 0;
  l -= i;

  return l;
}

// read the number inside the string
long Str2Int(const char *str, long default_result, int *error)
{
  char      *s;
  int        c, n, i, d[12], nd;
  enum {
    BEGINING,
    CHECKING,
    MINUS_FIRST,
    ZERO_FIRST,
    READING_DECIMAL,
    READING_HEXADECIMAL,
    READING_OCTAL,
    END_ZERO,
    END_DECIMAL,
    END_HEXADECIMAL,
    END_OCTAL
  } state;

  if(!str) return 0; // invalid parameter

  if(error) *error = 1;

  state = BEGINING;
  s     = (char *)str;
  n     = nd = 0;
  i     = 1;
  while((c = *(s++)))
  {
    switch(state)
    {
      case BEGINING: // removing spaces at the begining
        if(ISSPACE(c)) break;
        else           state = CHECKING;
      
      case CHECKING: // checking number format
        if     (c == '0')     state = ZERO_FIRST;    // begin with zero, can be a hexadecimal number
        else if(ISDIGIT(c)) { state = READING_DECIMAL; goto start_decimal; }
        else if(c == '-')     state = MINUS_FIRST;
        else                  return default_result; // invalid character
        break;

      case ZERO_FIRST:
        if(c == 'x' || c == 'X')        state = READING_HEXADECIMAL;               // hexadecimal number
        else if(c >= '0' && c <= '7') { state = READING_OCTAL; goto start_octal; } // octal number
        else if(ISSPACE(c))             state = END_ZERO;                          // zero
        else                            return default_result;                     // invalid character
        break;

      case MINUS_FIRST:
        if(!ISSPACE(c)) // suport for spaces between the - and the number
        {
          if(ISDIGIT(c)) { state = READING_DECIMAL; i = -1; goto start_decimal; } // negative decimal number
          else             return default_result;                                 // invalid character
        }
        break;

      case READING_DECIMAL: // reading a decimal integer
        if(ISDIGIT(c))
        {
          if(nd > 9) return default_result;        // number is too large
          start_decimal:
          d[nd++] = c - '0';
        }
        else if(ISSPACE(c)) state = END_DECIMAL;   // end
        else                return default_result; // invalid character
        break;

      case READING_HEXADECIMAL: // reading hexadecimal integer
        if(ISDIGIT(c))
        {
          if(nd > 7) return default_result; // number is too large
          d[nd++] = c - '0';
        }
        else if(c >= 'A' && c <= 'F')
        {
          if(nd > 7) return default_result; // number is too large
          d[nd++] = c - ('A' - 10);
        }
        else if(c >= 'a' && c <= 'f')
        {
          if(nd > 7) return default_result; // number is too large
          d[nd++] = c - ('a' - 10);
        }
        else if(ISSPACE(c)) state = END_HEXADECIMAL; // end
        else                return default_result;   // invalid character
        break;

      case READING_OCTAL: // reading octal
        if(c >= '0' && c <= '7')
        {
          if(nd > 12) return default_result; // number is too large
          start_octal:
          d[nd++] = c - '0';
        }
        else if(ISSPACE(c)) state = END_OCTAL;     // end
        else                return default_result; // invalid character
        break;

      case END_ZERO:
      case END_DECIMAL:
      case END_HEXADECIMAL:
      case END_OCTAL:
        if(!ISSPACE(c)) return default_result;
        break;
    }
  }

  // converting the number
  switch(state)
  {
    case ZERO_FIRST:
    case END_ZERO:
      n = 0;
      break;
    case READING_DECIMAL:
    case END_DECIMAL:
      for(--nd; nd >= 0; nd--)
      {
        n += d[nd] * i;
        i *= 10;
      }
      break;
    case READING_HEXADECIMAL:
    case END_HEXADECIMAL:
      for(--nd; nd >= 0; nd--)
      {
        n += d[nd] * i;
        i *= 16;
      }
      break;
    case READING_OCTAL:
    case END_OCTAL:
      for(--nd; nd >= 0; nd--)
      {
        n += d[nd] * i;
        i *= 8;
      }
      break;
    default:
      return default_result;
  }

  if(error) *error = 0;

  return n;
}
