
#ifndef _VDP_H
#define _VDP_H

#include "z80.h"
#include "def.h"

#ifdef __cplusplus
extern "C" {
#endif

extern const unsigned char VDP_HCounterTable[0x0200];
extern const unsigned char VDP_VCounterTable[0x0200];

struct _VDP {
  unsigned char  VRAM[0x4000];
  unsigned char  CRAM[0x40];
  unsigned char  Reg [0x10];
  unsigned char  Status;
  unsigned char  Latch;
  unsigned char  Pending;
  unsigned char  Buffer;
  unsigned char  Code;
  unsigned char  Limit;
  unsigned short Addr;
  int            ntab;
  int            satb;
  int            Line;
  int            Left;
} extern VDP;

void VDP_Init(void);
void VDP_CTRLWrite(int Value);
void VDP_DataWrite(int Value);
void VDP_Run(void);

extern unsigned char vdp_temp;
extern int           vdp_pixel;

#define VDP_CTRLRead()                 \
  vdp_temp    = VDP.Status,            \
  VDP.Pending = 0,                     \
  VDP.Status &= ~(0x80 | 0x40 | 0x20), \
  vdp_temp

#define VDP_DataRead()                       \
  vdp_temp    = VDP.Buffer,                  \
  VDP.Pending = 0,                           \
  VDP.Buffer  = VDP.VRAM[VDP.Addr & 0x3FFF], \
  VDP.Addr    = (VDP.Addr + 1) & 0x3FFF,     \
  vdp_temp

#define VDP_GetHCounter()                                \
  vdp_pixel = ((Z80.ICount % CYCLES_PER_LINE) >> 2) * 6, \
  VDP_HCounterTable[(vdp_pixel >> 1) & 0x1FF]

#define VDP_GetVCounter() VDP_VCounterTable[VDP.Line & 0x1FF]

#ifdef __cplusplus
}
#endif

#endif // _VDP_H
