
unsigned char *RdMap[8];
unsigned char *WrMap[8];

INLINE void WrZ80(word Addr, byte Value)
{
  WrMap[Addr >> 13][Addr & 0x1FFF] = Value;
  if(Addr >= 0xFFFC) System_MapZ80Mem(Addr, Value);
}

INLINE byte RdZ80(word Addr) { return RdMap[Addr >> 13][Addr & 0x1FFF]; }

INLINE void OutZ80(word Port, byte Value)
{
  switch(Port & 0xFF)
  {
    case 0xBE:
      VDP_DataWrite(Value);
      return;

    case 0xBD:
    case 0xBF:
      VDP_CTRLWrite(Value);
      return;

    case 0x7E:
    case 0x7F:
      PSG_Write(Value);
      return;

    case 0x06:
      if(System.Flags & SF_GAME_GEAR) PSG_GGStereoWrite(Value);
      return;

    case 0xF0:
      System.YM2413_Latch = Value & 0x3F;
      return;

    case 0xF1:
      if(EnableYM2413) YM2413_WriteReg(System.YM2413_Latch, Value);
      return;

    case 0x3F:
      System.Port3F = ((Value & 0x80) | (Value & 0x20) << 1) & 0xC0;
      return;

    case 0xF2:
      System.PortF2 = Value & 1;
      return;

    case 0xFF:
      if(System.Flags & SF_BIOS) System_Reset(FALSE);
      return;
  }
}

INLINE byte InZ80(word Port)
{
  switch(Port)
  {
    case 0xBD:
    case 0xBF:
      return VDP_CTRLRead();

    case 0xBE:
      return VDP_DataRead();

    case 0x7E:
      return VDP_GetVCounter();
    case 0x7F:
      return VDP_GetHCounter();

    case 0xC0:
    case 0xDC:
    {
      byte temp = 0xFF;
      if(Joypad[0].Up)    temp &= ~0x01;
      if(Joypad[0].Down)  temp &= ~0x02;
      if(Joypad[0].Left)  temp &= ~0x04;
      if(Joypad[0].Right) temp &= ~0x08;
      if(Joypad[0].One)   temp &= ~0x10;
      if(Joypad[0].Two)   temp &= ~0x20;
      if(!(System.Flags & SF_GAME_GEAR))
      {
        if(Joypad[1].Up)    temp &= ~0x40;
        if(Joypad[1].Down)  temp &= ~0x80;
      }
      return temp;
    }

    case 0xC1:
    case 0xDD:
    {
      byte temp = 0xFF;
      if(!(System.Flags & SF_GAME_GEAR))
      {
        if(Joypad[1].Left)  temp &= ~0x01;
        if(Joypad[1].Right) temp &= ~0x02;
        if(Joypad[1].One)   temp &= ~0x04;
        if(Joypad[1].Two)   temp &= ~0x08;
        if(Joypad[0].Reset || Joypad[1].Reset) temp &= ~0x10;
      }
      return (temp & 0x3F) | (System.Port3F & 0xC0);
    }

    case 0x00:
      if(System.Flags & SF_GAME_GEAR) return (Joypad[0].Start) ? 0x40 : 0x80 | 0x40;
      return 0x00;

    case 0x01:
    case 0x02:
    case 0x03:
    case 0x04:
    case 0x05:
      return 0x00;

    case 0xF2:
      return EnableYM2413 ? System.PortF2 : 0;
  }

  return 0xFF;
}
